/*-----------------------------------------------------------------------------------------------*\
\*-----------------------------------------------------------------------------------------------*/
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/spinlock.h>
#include <asm/fcntl.h>
#include <asm/ioctl.h>
#include <asm/errno.h>
#include <asm/uaccess.h>

#include <asm/mach_avm.h>
#include <asm/mips-boards/ar7.h>
#include <asm/mach-ar7/hw_gpio.h>
#include <asm/mach-ar7/hw_reset.h>

#define AR7GPIO_DEBUG

#if defined(AR7GPIO_DEBUG)
#define DBG(...)  printk(KERN_INFO __VA_ARGS__)
#else /*--- #if defined(AR7GPIO_DEBUG) ---*/
#define DBG(...)  
#endif /*--- #else ---*/ /*--- #if defined(AR7GPIO_DEBUG) ---*/

#if defined (CONFIG_MIPS_AR7)
extern union _hw_non_reset *AR7_hw_non_reset;
extern struct _hw_gpio *AR7_hw_gpio;
static spinlock_t ar7_gpio_spinlock;

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar7_gpio_init(void) {

    printk("[ar7_gpio_init]\n");
    AR7_hw_non_reset->Bits.gpio_unreset = 1;
    spin_lock_init(&ar7_gpio_spinlock);
    ar7_take_device_out_of_reset(GPIO_RESET_BIT);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int ar7_gpio_ctrl(unsigned int gpio_pin, enum _hw_gpio_function pin_mode, enum _hw_gpio_direction pin_dir) {
    unsigned int flags;

    if(gpio_pin >= 32)
        return(-1);

    spin_lock_irqsave(&ar7_gpio_spinlock, flags);
    if(pin_mode == GPIO_PIN) {
        AR7_hw_gpio->Enable.Register |= (1 << gpio_pin);
        if (pin_dir == GPIO_INPUT_PIN)
            AR7_hw_gpio->Direction.Register |= (1 << gpio_pin);
        else
            AR7_hw_gpio->Direction.Register &= ~(1 << gpio_pin);

    } else { /* FUNCTIONAL PIN */
        AR7_hw_gpio->Enable.Register &= ~(1 << gpio_pin);
    }
    spin_unlock_irqrestore(&ar7_gpio_spinlock, flags);
    return (0);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int ar7_gpio_out_bit(unsigned int gpio_pin, int value) {
    unsigned int flags;

    if(gpio_pin >= 32)
        return(-1);
    
    spin_lock_irqsave(&ar7_gpio_spinlock, flags);
    if(value == 1)
        AR7_hw_gpio->OutputData.Register |= 1 << gpio_pin;
    else
        AR7_hw_gpio->OutputData.Register &= ~(1 << gpio_pin);
    spin_unlock_irqrestore(&ar7_gpio_spinlock, flags);

    return(0);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int ar7_gpio_in_bit(unsigned int gpio_pin) {
    if(gpio_pin >= 32)
        return(-1);
    
    return (AR7_hw_gpio->InputData.Register & (1 << gpio_pin));
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int ar7_gpio_in_value(void) {
    return AR7_hw_gpio->InputData.Register;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar7_gpio_set_bitmask(unsigned int mask, unsigned int value) {
    unsigned int flags;
    spin_lock_irqsave(&ar7_gpio_spinlock, flags);
    AR7_hw_gpio->OutputData.Register &=  ~mask;
    AR7_hw_gpio->OutputData.Register |=  value & mask;
    spin_unlock_irqrestore(&ar7_gpio_spinlock, flags);
}

EXPORT_SYMBOL(ar7_gpio_ctrl);
EXPORT_SYMBOL(ar7_gpio_out_bit);
EXPORT_SYMBOL(ar7_gpio_in_bit);
EXPORT_SYMBOL(ar7_gpio_in_value);
EXPORT_SYMBOL(ar7_gpio_set_bitmask);

#endif /*--- #if defined (CONFIG_MIPS_AR7) ---*/
