#ifndef __CTRL_OLAY_SWIF_H__
#define __CTRL_OLAY_SWIF_H__ 1

// Texas Instruments Inc. 1999.  All rights reserved
// *****************************************************************************
// olaydp_interface_u.h
// This file contains the definitions and function prototypes for
// code overlays.
// History:
// --/--/-- Jack Huang      written
// 12/13/99 Michael Seidl   changed base address for full-rate code
// 01/21/00 Fred Wagner     derfmake changes
// 05/01/00 Barnett         Change OlayDP_DmemBaseAddr = 0x8000ec00 to reflect
//                          change in DMEM allocation.  Change also reflected
//                          in lnk_cpe.cmd.
// 04/12/00  Jack Huang     Code cleanup
// 06/16/00  Michael Seidl  added code for overlays through EMIF
// 10/25/00  Michael Seidl  moved overlay logic into seperate states
// 10/30/00  Michael Seidl  moved overlay logic from the statemachine into separate functions
// 11/01/00  Michael Seidl  added OLAYDP_2STEP
// 11/07/00  Michael Seidl  removed redundant code overlay related variables.(Waitframe, etc)
// 04/03/01  Michael Seidl  added a separate HOST_XFER_COMPLETE and DMA_XFER_COMPLETE to help debugging
// 04/03/01  J. Bergsagel   Removed OLAYDP_HOST token and related logic
// 11/11/02  Sameer V       Code clean up for Ax7 overlays. Removed unused enums and defines.
// 01/21/03  MCB            Implemented Ax7 UNIT-MODULE modular software framework.
// 03/08/04  Sameer         Added enumerations for overlay pages
// 04/15/04  Umesh Iyer     Added 2 new enumerations for training overlay pages. PAGE8_TRAINING1, PAGE9_TRAINING2
// 03/26/06  Venkat R       CQ9583: Added an Enum MAX_OVERLAY_PAGES and a constant RESET_ALLOTHERPAGES
//                          for new overlay manager features
// 03/31/06  Venkat R       UR8 Port: Converting olay typedef to a #define to facilitate its use in assembly
// 07/20/06  Brian L        Added PAGE14_OLAY
// 09/07/06  Brian L        CQ10780: Define olay page and olay segment parameters for new overlay scheme
// 09/21/06  Brian L        CQ11058: Added more segments since large segments are broken down to smaller sizes
//
// 10/10/06  Venkat R       CQ10811: Increase the page number to 28
// 10/26/06  Venkat R       CQ10811: Increase the segments  to 150
// 11/16/06  E Yin          CQ10772: Increase the segments to 160
// 02/08/07  Brian Lai      CQ11357: Increase overlay pages and segments for background
// *****************************************************************************

// States for the overlay transfer
enum {OLAYDP_NO_XFER_STARTED,
      OLAYDP_DMA_XFER_STARTED,
      OLAYDP_DMA_XFER_COMPLETE};

#define MAX_OVERLAY_PAGES           35
#define MAX_OVERLAY_SEGMENTS       170
#define MAX_SEGMENTS_PER_OLAY_PAGE  32

#define PAGE0_TEXT              0         // Page 0 is .text
#define PAGE1_OLAY              1
#define PAGE2_OLAY              2
#define PAGE3_OLAY              3
#define PAGE4_OLAY              4
#define PAGE5_OLAY              5
#define PAGE6_OLAY              6
#define PAGE7_OLAY              7
#define PAGE8_OLAY              8
#define PAGE9_OLAY              9
#define PAGE10_OLAY             10
#define PAGE11_OLAY             11
#define PAGE12_OLAY             12
#define PAGE13_OLAY             13
#define PAGE14_OLAY             14
#define PAGE15_OLAY             15
#define PAGE16_OLAY             16
#define PAGE17_OLAY             17
#define PAGE18_OLAY             18
#define PAGE19_OLAY             19
#define PAGE20_OLAY             20
#define PAGE21_OLAY             21
#define PAGE22_OLAY             22
#define PAGE23_OLAY             23
#define PAGE24_OLAY             24
#define PAGE25_OLAY             25
#define PAGE26_OLAY             26
#define PAGE27_OLAY             27
#define PAGE28_OLAY             28
#define PAGE29_OLAY             29
#define PAGE30_OLAY             30
#define PAGE31_OLAY             31
#define PAGE32_OLAY             32

                                          // Seg# 0 is a dummy segment
#define SEG01_OLAY              1
#define SEG02_OLAY              2
#define SEG03_OLAY              3
#define SEG04_OLAY              4
#define SEG05_OLAY              5
#define SEG06_OLAY              6
#define SEG07_OLAY              7
#define SEG08_OLAY              8
#define SEG09_OLAY              9
#define SEG10_OLAY              10
#define SEG11_OLAY              11
#define SEG12_OLAY              12
#define SEG13_OLAY              13
#define SEG14_OLAY              14
#define SEG15_OLAY              15
#define SEG16_OLAY              16
#define SEG17_OLAY              17
#define SEG18_OLAY              18
#define SEG19_OLAY              19
#define SEG20_OLAY              20
#define SEG21_OLAY              21
#define SEG22_OLAY              22
#define SEG23_OLAY              23
#define SEG24_OLAY              24
#define SEG25_OLAY              25
#define SEG26_OLAY              26
#define SEG27_OLAY              27
#define SEG28_OLAY              28
#define SEG29_OLAY              29
#define SEG30_OLAY              30
#define SEG31_OLAY              31
#define SEG32_OLAY              32
#define SEG33_OLAY              33
#define SEG34_OLAY              34
#define SEG35_OLAY              35
#define SEG36_OLAY              36
#define SEG37_OLAY              37
#define SEG38_OLAY              38
#define SEG39_OLAY              39
#define SEG40_OLAY              40
#define SEG41_OLAY              41
#define SEG42_OLAY              42
#define SEG43_OLAY              43
#define SEG44_OLAY              44
#define SEG45_OLAY              45
#define SEG46_OLAY              46
#define SEG47_OLAY              47
#define SEG48_OLAY              48
#define SEG49_OLAY              49
#define SEG50_OLAY              50
#define SEG51_OLAY              51
#define SEG52_OLAY              52
#define SEG53_OLAY              53
#define SEG54_OLAY              54
#define SEG55_OLAY              55
#define SEG56_OLAY              56
#define SEG57_OLAY              57
#define SEG58_OLAY              58
#define SEG59_OLAY              59
#define SEG60_OLAY              60
#define SEG61_OLAY              61
#define SEG62_OLAY              62
#define SEG63_OLAY              63
#define SEG64_OLAY              64
#define SEG65_OLAY              65
#define SEG66_OLAY              66
#define SEG67_OLAY              67
#define SEG68_OLAY              68
#define SEG69_OLAY              69
#define SEG70_OLAY              70
#define SEG71_OLAY              71
#define SEG72_OLAY              72
#define SEG73_OLAY              73
#define SEG74_OLAY              74
#define SEG75_OLAY              75
#define SEG76_OLAY              76
#define SEG77_OLAY              77
#define SEG78_OLAY              78
#define SEG79_OLAY              79
#define SEG80_OLAY              80
#define SEG81_OLAY              81
#define SEG82_OLAY              82
#define SEG83_OLAY              83
#define SEG84_OLAY              84
#define SEG85_OLAY              85
#define SEG86_OLAY              86
#define SEG87_OLAY              87
#define SEG88_OLAY              88
#define SEG89_OLAY              89
#define SEG90_OLAY              90
#define SEG91_OLAY              91
#define SEG92_OLAY              92
#define SEG93_OLAY              93
#define SEG94_OLAY              94
#define SEG95_OLAY              95
#define SEG96_OLAY              96
#define SEG97_OLAY              97
#define SEG98_OLAY              98
#define SEG99_OLAY              99
#define SEG100_OLAY             100
#define SEG101_OLAY             101
#define SEG102_OLAY             102
#define SEG103_OLAY             103
#define SEG104_OLAY             104
#define SEG105_OLAY             105
#define SEG106_OLAY             106
#define SEG107_OLAY             107
#define SEG108_OLAY             108
#define SEG109_OLAY             109
#define SEG110_OLAY             110
#define SEG111_OLAY             111
#define SEG112_OLAY             112
#define SEG113_OLAY             113
#define SEG114_OLAY             114
#define SEG115_OLAY             115
#define SEG116_OLAY             116
#define SEG117_OLAY             117
#define SEG118_OLAY             118
#define SEG119_OLAY             119
#define SEG120_OLAY             120
#define SEG121_OLAY             121
#define SEG122_OLAY             122
#define SEG123_OLAY             123
#define SEG124_OLAY             124
#define SEG125_OLAY             125
#define SEG126_OLAY             126
#define SEG127_OLAY             127
#define SEG128_OLAY             128
#define SEG129_OLAY             129
#define SEG130_OLAY             130
#define SEG131_OLAY             131
#define SEG132_OLAY             132
#define SEG133_OLAY             133
#define SEG134_OLAY             134
#define SEG135_OLAY             135
#define SEG136_OLAY             136
#define SEG137_OLAY             137
#define SEG138_OLAY             138
#define SEG139_OLAY             139
#define SEG140_OLAY             140
#define SEG141_OLAY             141
#define SEG142_OLAY             142
#define SEG143_OLAY             143
#define SEG144_OLAY             144
#define SEG145_OLAY             145
#define SEG146_OLAY             146
#define SEG147_OLAY             147
#define SEG148_OLAY             148
#define SEG149_OLAY             149
#define SEG150_OLAY             150
#define SEG151_OLAY             151
#define SEG152_OLAY             152
#define SEG153_OLAY             153
#define SEG154_OLAY             154
#define SEG155_OLAY             155
#define SEG156_OLAY             156
#define SEG157_OLAY             157
#define SEG158_OLAY             158
#define SEG159_OLAY             159
#define SEG160_OLAY             160
#define SEG161_OLAY             161
#define SEG162_OLAY             162
#define SEG163_OLAY             163
#define SEG164_OLAY             164
#define SEG165_OLAY             165
#define SEG166_OLAY             166
#define SEG167_OLAY             167
#define SEG168_OLAY             168
#define SEG169_OLAY             169
#define SEG170_OLAY             170

extern volatile unsigned int CTRL_olayviolation;
#endif // __CTRL_OLAY_SWIF_H__





