
/**
 *  \file   ddc_dsl_callback.h
 *
 *  \author Copyright (C) 2006 Texas Instruments Incorporated.
 *
 *  \brief  This file defines DSL Driver Core Callback APIs, which it needs from the DDA.
 *  
 *  \date   9/27/2006  AV      Created.
 *          10/30/2006 AV      CQ11118:Added DDA_sleep_interruptible().
 */

#ifndef _DDC_DSL_CB_H_
#define _DDC_DSL_CB_H_

#include "ddc_dsl_api.h"

/* Potentially to be mapped to PAL functions in the DDA. */
extern void *DDA_malloc(unsigned int size);
extern void *DDA_vmalloc(unsigned int size);
extern void  DDA_free(void *mem);
extern void  DDA_vfree(void *mem);
extern int   DDA_atoi(char *p);
extern int   DDA_atoh(char *p);
extern int   DDA_strcmp(const char *String1, const char *String2);
extern int   DDA_strncmp(const char *String1, const char *String2, int size);
extern unsigned int DDA_strlen(const char *String);
extern char* DDA_strcpy(char *String1, const char *String2);
extern void *DDA_memcpy(int * dst, int const * src, int len);
extern int DDA_memcmp(int * dst, int const * src, int len);
extern void *DDA_memset(void *Dest, int val, unsigned int size);
extern char *DDA_strstr(const char *String1, const char *String2);
extern int   DDA_toupper(int i);
extern int   DDA_printf(const char *Format, ...);
extern void  DDA_mdelay(unsigned long delay);
extern void  DDA_critical_on(void);
extern void  DDA_critical_off(void);
extern int  DDA_sprintf ( char * buffer, const char * format, ...); 
extern int  DDA_snprintf ( char * buffer, unsigned int len, const char * format, ...); 
extern unsigned long DDA_strtoul(const char *str, char **endptr, int base);

//Functions DDA Functions needed
extern int DDA_atm_receive (void *os_dev, int ch, unsigned int packet_size, void *os_receive_info, void *data);
#ifdef UR8_SAR
extern void DDA_free_buffer(void *OSRecieveInfo, void *memptr);
extern void DDC_free_rxbuffer(void *hDDA, void *dataToken, unsigned int channel);
#endif
#ifdef AR7_SAR
extern void DDA_free_buffer(OS_RECEIVEINFO *OSRecieveInfo, void *memptr);
#endif

extern void DDA_free_dev(void *mem);
extern void DDA_free_dma_transfer(void *mem);

#ifdef UR8_SAR
extern void * DDA_malloc_rxbuffer(unsigned int size, void *mem_base, unsigned int mem_range, void *osSetup,
                       void *hal_info, void**os_receive_info, void *os_dev);
extern void *DDC_malloc_rxbuffer(void *hDDA, void *dataToken, unsigned int size, unsigned int channel);
#endif
#ifdef AR7_SAR
extern void * DDA_malloc_rxbuffer(unsigned int size, void *mem_base, unsigned int mem_range, void *osSetup,
                       HAL_RECEIVEINFO *hal_info, OS_RECEIVEINFO **os_receive_info, void *os_dev);
#endif

extern void *DDA_malloc_dev(unsigned int size);
extern void *DDA_malloc_dma_transfer(unsigned int size, void *base, unsigned int memrange);

#ifdef UR8_SAR
extern int DDC_send_complete(void* hDDA, void *netDataTokens, int numTokens, unsigned int channel);
extern int DDA_send_complete(void* OsSendInfo, int count);
#endif
#ifdef AR7_SAR
extern int  DDA_send_complete(OS_SENDINFO *OsSendInfo, int count);
#endif

extern int  DDA_request_interrupt_pacing(ti_dsl_int_t intr, unsigned int pace_val);


extern int DDA_Led_func(int state_id);

extern int DDA_sleep_interruptible(unsigned long millisecs);


#ifdef DEBUG_BUILD
extern void xadump(unsigned char *buff, int len, int debugLev);
#define xdump(buff, len, debugLev) xadump(buff, len, debugLev)
#else
#define xdump(buff, len, debugLev)
#endif

#ifdef DEBUG_BUILD
extern void DDA_dprintf( int uDbgLevel, char * szFmt, ...);
#define dgprintf DDA_dprintf
#else
#define dgprintf(uDbgLevel, szFmt, args...)
#endif


#endif /* _DDC_DSL_CB_H_ */
