/*------------------------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*------------------------------------------------------------------------------------------*/
#include <stdarg.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timer.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/reboot.h>
#if !defined(CONFIG_NETCHIP_ADM69961)
#define CONFIG_NETCHIP_ADM69961
#endif
#include <linux/avm_event.h>
#include <linux/avm_led.h>
#include <asm/mips-boards/ohio.h>
#include <asm/mach_avm.h>

#include "cpmac_if.h"
#include "cpmac_const.h"
#include "cpmac_debug.h"
#include "cpmac_main.h"
#include "cpmac_reg.h"
#include "cpphy_const.h"
#include "cpphy_types.h"
#include "cpphy_mdio.h"
#include "cpphy_cppi.h"
#include "cpphy_main.h"
#include "cpphy_if.h"
#include "cpphy_adm6996.h"


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static struct notifier_block reboot_notifier_block = { NULL, NULL, 0};

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_main_mac_address_set(cpphy_global_t *cpphy_global, struct net_device *p_dev) {
#   if defined(CONFIG_MIPS_OHIO)
    /* high */
    *((volatile unsigned int *)(p_dev->base_addr + 0x508)) = 0;
    /* med */
    *((volatile unsigned int *)(p_dev->base_addr + 0x504)) =   (p_dev->dev_addr[0] <<  0)
                                                             | (p_dev->dev_addr[1] <<  8)
                                                             | (p_dev->dev_addr[2] << 16)
                                                             | (p_dev->dev_addr[3] << 24);
    /* low */
    *((volatile unsigned int *)(p_dev->base_addr + 0x500)) =   (1 << 20)  /*--- funktion unbekannt ---*/
                                                             | (1 << 19)  /*--- funktion unbekannt ---*/
                                                             | (0 << 16)  /*--- funktion unbekannt ---*/
                                                             | (p_dev->dev_addr[4] <<  0)
                                                             | (p_dev->dev_addr[5] <<  8);
#   elif defined(CONFIG_MIPS_AR7) /*--- #if defined(CONFIG_MIPS_OHIO) ---*/
    CPMAC_MACADDRLO_0(p_dev->base_addr) = p_dev->dev_addr[5];
    CPMAC_MACADDRMID(p_dev->base_addr)  = p_dev->dev_addr[4];
    CPMAC_MACADDRHI(p_dev->base_addr)   =    p_dev->dev_addr[0] 
                                          | (p_dev->dev_addr[1] <<  8)
                                          | (p_dev->dev_addr[2] << 16) 
                                          | (p_dev->dev_addr[3] << 24);
#   else /*--- #if defined(CONFIG_MIPS_OHIO) ---*/
#   warning "No code for setting the MAC address!";
#   endif /*--- #else ---*/ /*--- #if defined(CONFIG_MIPS_OHIO) ---*/
}


/*----------------------------------------------------------------------------------*\
\*----------------------------------------------------------------------------------*/
static void cpphy_main_config_apply (cpphy_global_t *cpphy_global, struct net_device *p_dev)
{
  cpmac_priv_t *cpmac_priv = (cpmac_priv_t *) netdev_priv (p_dev);
  cpmac_capabilities_t *capabilities = &cpmac_priv->capabilities;
  unsigned int RxMbpEnable;

  CPMAC_RX_MAXLEN(p_dev->base_addr) = CPPHY_MAX_RX_BUFFER_SIZE;

  cpphy_main_mac_address_set (cpphy_global, p_dev);

  /* Init RX Multicast/Broadcast/Promiscous */
  RxMbpEnable = 0;
  if (capabilities->promiscous) {
    RxMbpEnable |= RX_CAF_EN;
  }
  if (capabilities->broadcast) {
    RxMbpEnable |= RX_BROAD_EN;
  }
  if (capabilities->multicast) {
    RxMbpEnable |= RX_MULT_EN;
  }
  if (capabilities->short_frames) {
    RxMbpEnable |= RX_CSF_EN;
  }
  if (!capabilities->long_frames) {
    RxMbpEnable |= RX_NO_CHAIN;
  }
  CPMAC_RX_MBP_ENABLE (p_dev->base_addr) = RxMbpEnable;
  DEB_TRC("cpphy_main_config_apply, MBP_ENABLE 0x%08X\n", 
      CPMAC_RX_MBP_ENABLE (p_dev->base_addr));

  CPMAC_MACHASH1 (p_dev->base_addr) = capabilities->multicast ? 0xffffffff : 0;
  CPMAC_MACHASH2 (p_dev->base_addr) = capabilities->multicast ? 0xffffffff : 0;

  CPMAC_RX_UNICAST_CLEAR(p_dev->base_addr) = 0xff;
  /* channel 0 for unicast */
  CPMAC_RX_UNICAST_SET(p_dev->base_addr) = 1;

  /* Enable MII and flow control in both directions */
  /*--- CPMAC_MACCONTROL(p_dev->base_addr) |= MII_EN | TX_FLOW_EN; ---*/
  CPMAC_MACCONTROL(p_dev->base_addr) |= MII_EN | TX_FLOW_EN | RX_FLOW_EN;
}

/*----------------------------------------------------------------------------------*\
\*----------------------------------------------------------------------------------*/
static void cpphy_main_close_cpmac (struct net_device *p_dev)
{
  /* perform normal close duties */
  CPMAC_MACCONTROL(p_dev->base_addr) &= ~MII_EN;
  CPMAC_TX_CONTROL(p_dev->base_addr) &= ~TX_EN;
  CPMAC_RX_CONTROL(p_dev->base_addr) &= ~RX_EN;

  /* disable interrupt masks */
  CPMAC_TX_INTMASK_CLEAR(p_dev->base_addr) = 0xFF;
  CPMAC_RX_INTMASK_CLEAR(p_dev->base_addr) = 0xFF;
}


/*----------------------------------------------------------------------------------*\
\*----------------------------------------------------------------------------------*/
cpmac_err_t cpphy_main_open (cpphy_global_t *cpphy_global, struct net_device *p_dev)
{
  unsigned int i, priority;
  cpmac_err_t ret = CPMAC_ERR_NOERR;
  /*--- cpmac_priv_t *cpmac_priv = (cpmac_priv_t *) netdev_priv (p_dev); ---*/

  /* Decide, which TxChannel to use for transfer */
  cpphy_global->cppi.TxChannel = 0;

  /* After Reset clear the Transmit and Receive DMA Head Descriptor Pointers */

  for (i=0; i<8; i++) {
    CPMAC_TX_HDP (p_dev->base_addr, i) = 0;
  }
  for (i=0; i<8; i++) {
    CPMAC_RX_HDP (p_dev->base_addr, i) = 0;
  }

  CPMAC_RX_BUFFER_OFFSET (p_dev->base_addr) = 0;

  cpphy_global->cppi.TxPrioQueues.NumberOfPrioQueues = CPPHY_PRIO_QUEUES;

  for(priority = 0; priority < CPPHY_PRIO_QUEUES; priority++) {
      cpphy_global->cppi.TxPrioQueues.q[priority].MaxSize = 32 / (priority + 1);
      cpphy_global->cppi.TxPrioQueues.q[priority].Free = cpphy_global->cppi.TxPrioQueues.q[priority].MaxSize;
      cpphy_global->cppi.TxPrioQueues.q[priority].DMAFree = 3 * cpphy_global->cppi.TxPrioQueues.q[priority].MaxSize;
      cpphy_global->cppi.TxPrioQueues.q[priority].Pause = 0;
      cpphy_global->cppi.TxPrioQueues.q[priority].PauseInit = priority + 1;
      cpphy_global->cppi.TxPrioQueues.q[priority].First = NULL;
      cpphy_global->cppi.TxPrioQueues.q[priority].Last = NULL;
      DEB_INFO("Priority queue %u is size %u\n", priority, cpphy_global->cppi.TxPrioQueues.q[priority].MaxSize);
  }
  cpphy_global->cpmac_priv->TxPrioQueues = &cpphy_global->cppi.TxPrioQueues;

  /* Init Tx and Rx DMA */

  CPMAC_TX_CONTROL (p_dev->base_addr) |= TX_EN;
  CPMAC_RX_CONTROL (p_dev->base_addr) |= RX_EN;

  /* Enable HostErr Ints (only for debugging) */
  CPMAC_MAC_INTMASK_SET  (p_dev->base_addr) |= 2;

  cpphy_global->cppi.cpmac_priv = cpphy_global->cpmac_priv;
  cpphy_global->cpmac_priv->cppi = &cpphy_global->cppi;
  cpphy_global->cppi.mdio = &cpphy_global->mdio;
  cpphy_global->cppi.hw_state = CPPHY_HW_ST_OPENED;

  cpphy_mdio_init (cpphy_global);
  
  if(reboot_notifier_block.notifier_call == NULL) {
      reboot_notifier_block.notifier_call = cpphy_entry_prepare_reboot;
      reboot_notifier_block.next = NULL;
      reboot_notifier_block.priority = 0;
      register_reboot_notifier(&reboot_notifier_block);
  }

  cpphy_main_config_apply (cpphy_global, p_dev);  /* Apply Configuration Values to Device */
  return ret;
}

/*----------------------------------------------------------------------------------*\
\*----------------------------------------------------------------------------------*/
cpmac_err_t cpphy_main_close (cpphy_cppi_t *cppi)
{
  cpmac_err_t ret = CPMAC_ERR_NOERR;
  struct net_device *p_dev = (struct net_device *)cppi->cpmac_priv->owner;

  DEB_INFO("cpphy_main_close, init\n");

  /* Verify proper device state */
  if (cppi->hw_state == CPPHY_HW_ST_OPENED) {
    if(reboot_notifier_block.notifier_call) {
      unregister_reboot_notifier(&reboot_notifier_block);
      reboot_notifier_block.notifier_call = NULL;
    }

    cpphy_main_close_cpmac (p_dev);

    if(cppi->mdio->cpmac_switch) {
      /* FIXME */
    } else {
      if(cppi->cpmac_priv->led_handle[cppi->cpmac_priv->inst] != 0) {
        avm_led_free_handle (cppi->cpmac_priv->led_handle[cppi->cpmac_priv->inst]);
      }
    }

    /* put device back into reset */
    avm_put_device_into_reset (cppi->cpmac_priv->mac_reset_bit);
    msleep (2);

    cppi->hw_state = CPPHY_HW_ST_INITIALIZED;
  }

  return ret;
}

/*----------------------------------------------------------------------------------*\
\*----------------------------------------------------------------------------------*/
cpmac_err_t cpphy_main_register (cpphy_global_t *cpphy_global)
{
  cpmac_service_funcs_t service_funcs;

  service_funcs.init = &cpphy_if_init;
  service_funcs.deinit = &cpphy_if_deinit;
  service_funcs.control_req = &cpphy_if_control_req;
  service_funcs.data_to_phy = &cpphy_if_data_to_phy;
  service_funcs.isr_tasklet = &cpphy_if_isr_tasklet;
  service_funcs.isr_end = &cpphy_if_isr_end;

  /* register at mac */
  return cpmac_if_register (cpphy_global, &service_funcs);
}
EXPORT_SYMBOL(cpphy_main_register);

