/*------------------------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*------------------------------------------------------------------------------------------*/
#include <linux/interrupt.h>
#include <linux/timer.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/delay.h>
#include <asm/mach_avm.h>
#include <asm/mips-boards/ohio.h>
#include <asm/mips-boards/ohioint.h>

#if defined(CONFIG_MIPS_OHIO)
#include <asm/mach-ohio/hw_mdio.h>
#endif/*--- #if defined(CONFIG_MIPS_AR7) ---*/

#if defined(CONFIG_MIPS_AR7)
#include <asm/mach-ar7/hw_mdio.h>
#endif/*--- #if defined(CONFIG_MIPS_AR7) ---*/

#if defined(CONFIG_AVM_POWERMETER)
#include <linux/avm_power.h>
#endif /*--- #if defined(CONFIG_AVM_POWERMETER) ---*/

#if !defined(CONFIG_NETCHIP_ADM69961)
#define CONFIG_NETCHIP_ADM69961
#endif /*--- #if !defined(CONFIG_NETCHIP_ADM69961) ---*/
#include <linux/avm_event.h>
#include <linux/avm_led.h>
#include <linux/avm_cpmac.h>
#include <linux/adm_reg.h>

#include "cpmac_if.h"
#include "cpmac_const.h"
#include "cpmac_debug.h"
#include "cpmac_reg.h"
#include "cpphy_const.h"
#include "cpphy_types.h"
#include "cpphy_mdio.h"
#include "adm6996.h"
#include "cpphy_adm6996.h"

/* FIXME: Temporarily define this here, until the include file is available */
#define TNETD73XX_EPHY_TRIM_VALUE 0xAC


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_mdix_delay(cpphy_mdio_t *mdio) {
    if(mdio->nway_mode & NWAY_AUTOMDIX) {
        mdio->timeout = CPPHY_MDIX_TO + CPPHY_AUTOMDIX_DELAY_MIN / 10;
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_wait_for_access_complete(cpphy_mdio_t *mdio) {
    while(MDIO_USER_ACCESS(mdio->inst) & MDIO_USER_ACCESS_GO);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_user_access(cpphy_mdio_t *mdio, 
                                   unsigned int method, 
                                   unsigned int regadr, 
                                   unsigned int phyadr, 
                                   unsigned int data) {
    unsigned int  control;

    control = MDIO_USER_ACCESS_GO | (method) |
              MDIO_USER_ACCESS_REGADDR(regadr & 0x1f) |
              MDIO_USER_ACCESS_PHYADDR(phyadr & 0x1f) |
              MDIO_USER_ACCESS_DATA(data & 0xffff);
    MDIO_USER_ACCESS(mdio->inst) = control;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int cpphy_mdio_user_access_read(cpphy_mdio_t *mdio, 
                                         unsigned int  regadr, 
                                         unsigned int  phyadr) {
    cpphy_mdio_wait_for_access_complete(mdio);  /* Wait until UserAccess ready */
    cpphy_mdio_user_access(mdio, 0, regadr, phyadr, 0); /* no Write Bit */
    cpphy_mdio_wait_for_access_complete(mdio);  /* Wait for Read to complete */

    return (MDIO_USER_ACCESS(mdio->inst) & MDIO_USER_ACCESS_DATA(0xffff));
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void cpphy_mdio_user_access_write(cpphy_mdio_t *mdio, 
                                  unsigned int  regadr, 
                                  unsigned int  phyadr, 
                                  unsigned int  data) {
    cpphy_mdio_wait_for_access_complete(mdio);  /* Wait until UserAccess ready */
    cpphy_mdio_user_access(mdio, MDIO_USER_ACCESS_WRITE, regadr, phyadr, data);
}


/*------------------------------------------------------------------------------------------*\
 * Function Name : WriteEphyTestRegs                                                        *
 * Purpose       : Program EPHY registers                                                   *
 * Parameters    : PHY number, register ID, register value                                  *
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_WriteEphyTestRegs(cpphy_mdio_t *mdio, 
                                         unsigned int  MiiPhyNum, 
                                         unsigned int  TestCfgRegNum, 
                                         unsigned int  CfgValue) {
    /* This sequence will change the PHY to test mode */
    cpphy_mdio_user_access_write(mdio, 20, MiiPhyNum, 0x0000);
    cpphy_mdio_user_access_write(mdio, 20, MiiPhyNum, 0x0400);
    cpphy_mdio_user_access_write(mdio, 20, MiiPhyNum, 0x0000);
    cpphy_mdio_user_access_write(mdio, 20, MiiPhyNum, 0x0400);
  
    /* Write register value */
    cpphy_mdio_user_access_write(mdio, 23, MiiPhyNum, 0x8100 | (CfgValue & 0xFF));
    cpphy_mdio_user_access_write(mdio, 20, MiiPhyNum, 0x4400 | (TestCfgRegNum & 0xFF));
  
    /* Back to normal mode */
    cpphy_mdio_user_access_write(mdio, 20, MiiPhyNum, 0x0000);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_mdix_update(unsigned int On) {
    avm_gpio_out_bit(GPIO_BIT_MII_MDIX, On ? 1 : 0);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_timeout(cpphy_mdio_t *mdio) {
    if(mdio->nway_mode & NWAY_AUTOMDIX) {
        /* AutoMdix not supported */
        /* Indicate to Ticks() that MDI/MDIX mode switch is needed */
        mdio->mdix ^= CPPHY_MDIO_FLG_MDIX_ON;    /* toggle MDI / MDIX */
        DEB_INFO("cpphy_mdio_timeout, xchange mdix: %u\n", mdio->mdix);
        cpphy_mdio_mdix_update(mdio->mdix);

        /* Reset state machine to FOUND */
        mdio->state = CPPHY_MDIO_ST_FOUND;
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void cpphy_mdio_event_dataupdate(cpphy_mdio_t *mdio) {
    /* Update LAN-LED-state */
    /* Switch LED control happens in cpphy_adm6996.c */
    if(!mdio->cpmac_switch && (mdio->cpmac_priv->led_handle[mdio->cpmac_priv->inst] != 0)) {
        avm_led_action_with_handle(mdio->cpmac_priv->led_handle[mdio->cpmac_priv->inst], 
                                   (mdio->state == CPPHY_MDIO_ST_LINKED) ? avm_led_on 
                                                                         : avm_led_off);
    }
    if(mdio->cpmac_switch) {
        /* The data in the adm_serial_register got updated by a previous call to *\
        \* ADM_GET_SERIAL_REG (REG_ADM_LC_STATUS0)                               */

        mdio->event_notify |= mdio->event_data.port[0].link != adm_serial_register.status.port0linkup;
        mdio->event_notify |= mdio->event_data.port[1].link != adm_serial_register.status.port1linkup;
        mdio->event_notify |= mdio->event_data.port[2].link != adm_serial_register.status.port2linkup;
        mdio->event_notify |= mdio->event_data.port[3].link != adm_serial_register.status.port3linkup;
        mdio->event_notify |= mdio->event_data.port[4].link != adm_serial_register.status.port4linkup;
        if(mdio->event_notify) {
            mdio->event_data.port[0].link = adm_serial_register.status.port0linkup;
            mdio->event_data.port[0].speed100 = adm_serial_register.status.port0speed;
            mdio->event_data.port[0].fullduplex = adm_serial_register.status.port0duplex;
            mdio->event_data.port[1].link = adm_serial_register.status.port1linkup;
            mdio->event_data.port[1].speed100 = adm_serial_register.status.port1speed;
            mdio->event_data.port[1].fullduplex = adm_serial_register.status.port1duplex;
            mdio->event_data.port[2].link = adm_serial_register.status.port2linkup;
            mdio->event_data.port[2].speed100 = adm_serial_register.status.port2speed;
            mdio->event_data.port[2].fullduplex = adm_serial_register.status.port2duplex;
            mdio->event_data.port[3].link = adm_serial_register.status.port3linkup;
            mdio->event_data.port[3].speed100 = adm_serial_register.status.port3speed;
            mdio->event_data.port[3].fullduplex = adm_serial_register.status.port3duplex;
            mdio->event_data.port[4].link = adm_serial_register.status.port4linkup;
            mdio->event_data.port[4].speed100 = adm_serial_register.status.port4speed;
            mdio->event_data.port[4].fullduplex = adm_serial_register.status.port4duplex;
        }
    } else if(mdio->event_data.port[mdio->inst].link != (mdio->state == CPPHY_MDIO_ST_LINKED)) {
        /* Nothing to do for low phy when switch is connected to high cpmac */
        /* -> dont register low cpphy driver at cpmac */
        mdio->event_notify = 1;
        mdio->event_data.port[mdio->inst].link = (mdio->state == CPPHY_MDIO_ST_LINKED);
        mdio->event_data.port[mdio->inst].speed100 = !mdio->slow_speed;
        mdio->event_data.port[mdio->inst].fullduplex = !mdio->half_duplex;
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void cpphy_mdio_update(cpphy_mdio_t *mdio, struct cpmac_event_struct *event_struct) {
    if(mdio->cpmac_switch) {
        memcpy((unsigned char *) event_struct, 
               (unsigned char *) &mdio->event_data, 
               sizeof (struct cpmac_event_struct));
        event_struct->ports = 5; /* patch all switch ports */
    } else {
        event_struct->port[mdio->inst] = mdio->event_data.port[mdio->inst];
    }
}


/*------------------------------------------------------------------------------------------*\
  unsigned int rx_errors;
  unsigned int tx_errors;
  unsigned int collisions;
\*------------------------------------------------------------------------------------------*/
void cpphy_mdio_update_hw_status(cpphy_mdio_t *mdio, struct net_device_stats *stats) {
    if(mdio->cpmac_switch) {
        if(adm_serial_register.status.port0linkup) {
            adm_update_error_status(mdio, 0);
            stats->collisions += adm_serial_register.collisions.port0;
            stats->rx_errors += adm_serial_register.errors.port0;
        }
        if(adm_serial_register.status.port1linkup) {
            adm_update_error_status(mdio, 1);
            stats->collisions += adm_serial_register.collisions.port1;
            stats->rx_errors += adm_serial_register.errors.port1;
        }
        if(adm_serial_register.status.port2linkup) {
            adm_update_error_status(mdio, 2);
            stats->collisions += adm_serial_register.collisions.port2;
            stats->rx_errors += adm_serial_register.errors.port2;
        }
        if(adm_serial_register.status.port3linkup) {
            adm_update_error_status(mdio, 3);
            stats->collisions += adm_serial_register.collisions.port3;
            stats->rx_errors += adm_serial_register.errors.port3;
        }
        if(adm_serial_register.status.port4linkup) {
            adm_update_error_status(mdio, 4);
            stats->collisions += adm_serial_register.collisions.port4;
            stats->rx_errors += adm_serial_register.errors.port4;
        }
        adm_update_error_status(mdio, 5);
        stats->collisions += adm_serial_register.collisions.port5;
        stats->rx_errors += adm_serial_register.errors.port5;
        /* unfortunately switch has no distinction for rx/tx errors */
        stats->tx_errors = stats->rx_errors;
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_reset_phy(cpphy_mdio_t *mdio) {
    DEB_INFO("cpphy_mdio_reset_phy (%u)\n", mdio->phy_num);
    cpphy_mdio_user_access_write(mdio, PHY_CONTROL_REG, mdio->phy_num, PHY_RESET);

    /* Read control register until Phy Reset is complete */
    while(cpphy_mdio_user_access_read(mdio, PHY_CONTROL_REG, mdio->phy_num) & PHY_RESET);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_check_pdown_state(cpphy_mdio_t *mdio) {
    if(mdio->power_down && !(cpphy_mdio_user_access_read(mdio, PHY_GENERIC_STATUS_REG, mdio->phy_num) & 
                             PHY_STATUS_MD)) {
        cpphy_mdio_user_access_write(mdio, PHY_CONTROL_REG, mdio->phy_num, PHY_PDOWN);
        mdio->state = CPPHY_MDIO_ST_PDOWN;
    } else if(mdio->timeout) {
        mdio->timeout--;
    } else {
        DEB_INFO("cpphy_mdio_check_pdown_state, tmo\n");
        cpphy_mdio_timeout(mdio);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_finding_state(cpphy_mdio_t *mdio) {
    if(mdio->timeout) {
        mdio->timeout--;
    } else if(MDIO_ALIVE & (1 << mdio->phy_num)) {
        /*  Phy Found! */
        mdio->state = CPPHY_MDIO_ST_FOUND;
        /* powerdown for external phy */
        mdio->power_down = (cpphy_mdio_user_access_read(mdio, PHY_Identifier_1, mdio->phy_num) == 0x2e) && 
                           ((cpphy_mdio_user_access_read(mdio, PHY_Identifier_2, mdio->phy_num) & 0xfff0) == 0xcc60);
        DEB_INFO("cpphy_mdio_finding_state, phy_num: %u %u\n", mdio->phy_num, mdio->power_down);
    } else {
        DEB_WARN("cpphy_mdio_finding_state, timed out looking for a Phy (MDIO_ALIVE = %x)\n", MDIO_ALIVE);
    }
}


/*------------------------------------------------------------------------------------------*\
  This algo is almost according to spec of TMS320C6000 DSP
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_found_state(cpphy_mdio_t *mdio) {
    /* Reset the Phy and proceed with auto-negotiation */
    cpphy_mdio_reset_phy(mdio);

    /* Fix phy according to TI */
    if(!mdio->cpmac_switch) {
        cpphy_mdio_WriteEphyTestRegs(mdio, 31, 0x16, TNETD73XX_EPHY_TRIM_VALUE);
    }

    /* Now setup the MDIOUserPhySel register */
    MDIO_USER_PHY_SELECT(mdio->inst) = mdio->phy_num;

    /* only using NWAY compliant Phy */
    DEB_INFO("cpphy_mdio_found_state, NWAY\n");
    cpphy_mdio_user_access_write(mdio, NWAY_ADVERTIZE_REG, mdio->phy_num, (mdio->nway_mode & ~NWAY_AUTOMDIX));
    cpphy_mdio_user_access_write(mdio, PHY_CONTROL_REG, mdio->phy_num, AUTO_NEGOTIATE_EN);
    cpphy_mdio_user_access_write(mdio, PHY_CONTROL_REG, mdio->phy_num, AUTO_NEGOTIATE_EN|RENEGOTIATE);
    mdio->state = CPPHY_MDIO_ST_NWAY_START;
    mdio->timeout = CPPHY_NWST_TO;
    cpphy_mdio_mdix_delay(mdio);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_pdown_state(cpphy_mdio_t *mdio) {
    if(cpphy_mdio_user_access_read(mdio, PHY_GENERIC_STATUS_REG, mdio->phy_num) & PHY_STATUS_MD) {
        /* Reset state machine to FOUND */
        mdio->state = CPPHY_MDIO_ST_FOUND;
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_nway_start_state(cpphy_mdio_t *mdio) {
    unsigned int PhyMode;

    /* Wait for Negotiation to start */
    PhyMode = cpphy_mdio_user_access_read(mdio, PHY_CONTROL_REG, mdio->phy_num);

    /* auto negotiation has started */
    if(!(PhyMode & RENEGOTIATE)) {
        /*Flush pending latch bits*/
        cpphy_mdio_user_access_read(mdio, PHY_STATUS_REG, mdio->phy_num); 
        mdio->timeout = CPPHY_NWDN_TO;
        mdio->state = CPPHY_MDIO_ST_NWAY_WAIT;
        cpphy_mdio_mdix_delay(mdio);  /* If AutoMdix set new delay */
    } else if(mdio->timeout) {
        mdio->timeout--;
    } else {
        cpphy_mdio_timeout(mdio);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_nway_wait_state(cpphy_mdio_t *mdio) {
    unsigned int PhyStatus, NWAYREadvertise, NegMode, i, j;

    if(mdio->cpmac_switch) {
        mdio->state = CPPHY_MDIO_ST_LINK_WAIT;
        DEB_WARN("cpphy_mdio_nway_wait_state, should never get here\n");
    } else {
        PhyStatus = cpphy_mdio_user_access_read(mdio, PHY_STATUS_REG, mdio->phy_num);
        if(PhyStatus & NWAY_COMPLETE) {
            NWAYREadvertise = cpphy_mdio_user_access_read(mdio, NWAY_REMADVERTISE_REG, mdio->phy_num);

            /* Negotiated mode is what we and the remote have in common (limit to fields below) */
            NegMode = (mdio->nway_mode & NWAYREadvertise) & (NWAY_FD100|NWAY_HD100|NWAY_FD10|NWAY_HD10);

            DEB_INFO("cpphy_mdio_nway_wait_state, Phy %u, Status %#x, NegMode %#x, NWAYRE %#x\n",
                     mdio->phy_num, PhyStatus, NegMode, NWAYREadvertise);

            if(!NegMode) {
                /* or 10 ?? who knows, Phy is not MII compliant*/
                NegMode = mdio->nway_mode & (NWAY_HD100|NWAY_HD10); 
                DEB_WARN("cpphy_mdio_nway_wait_state, Negotiation w/o agreement, default is HD\n");
            }
            /* select 'highest' mode */
            for(j=0x8000, i=0; (i<16) && !(j&NegMode); i++, j>>=1);
            NegMode=j;

            if(NegMode == NWAY_FD100) {
                DEB_INFO("cpphy_mdio_nway_wait_state, FullDuplex 100 Mbps\n");
            } else if(NegMode == NWAY_HD100) {
                mdio->half_duplex = 1;
                DEB_INFO("cpphy_mdio_nway_wait_state, HalfDuplex 100 Mbps\n");
            } else if(NegMode == NWAY_FD10) {
                mdio->slow_speed = 1;
                DEB_INFO("cpphy_mdio_nway_wait_state, FullDuplex 10 Mbps\n");
            } else if(NegMode == NWAY_HD10) {
                mdio->slow_speed = 1;
                mdio->half_duplex = 1;
                DEB_INFO("cpphy_mdio_nway_wait_state, HalfDuplex 10 Mbps\n");
            } else {
                /* should never get here */
                DEB_ERR("cpphy_mdio_nway_wait_state, ill default nway mode\n");
            }
            if(PhyStatus & PHY_LINKED) {
                mdio->state = CPPHY_MDIO_ST_LINKED;
                cpphy_mdio_event_dataupdate(mdio); /* Status changed to LINKED */
            } else {
                mdio->state = CPPHY_MDIO_ST_LINK_WAIT;
                /* to check: tmo missing? */
            }
        } else {
            cpphy_mdio_check_pdown_state(mdio);
        }
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_link_wait_state(cpphy_mdio_t *mdio) {
    unsigned int PhyStatus;

    if(mdio->cpmac_switch) {
        if(mdio->linked) {
            DEB_TRC("cpphy_mdio_link_wait_state, adm status: %#x\n", mdio->adm_status);
            mdio->state = CPPHY_MDIO_ST_LINKED;
        }
    } else {
        PhyStatus = cpphy_mdio_user_access_read(mdio, PHY_STATUS_REG, mdio->phy_num);
        if(PhyStatus & PHY_LINKED) {
            mdio->state = CPPHY_MDIO_ST_LINKED;
            cpphy_mdio_event_dataupdate(mdio); /* Status changed to LINKED */
#           if defined(CONFIG_AVM_POWERMETER)
            PowerManagmentRessourceInfo(powerdevice_ethernet, 100);
#           endif /*--- #if defined(CONFIG_AVM_POWERMETER) ---*/
        } else {
            cpphy_mdio_check_pdown_state (mdio);
        }
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_linked_state(cpphy_mdio_t *mdio) {
    if(mdio->cpmac_switch) {
        if(!mdio->linked) {
            mdio->state = CPPHY_MDIO_ST_LINK_WAIT;
        }
    } else if(!(MDIO_LINK & (1 << mdio->phy_num))) {
        /* No switch, not linked */
        if(mdio->nway_mode & NWAY_AUTO) {
            mdio->state = CPPHY_MDIO_ST_NWAY_WAIT;
            mdio->timeout = CPPHY_NWDN_TO;
        } else {
            mdio->state = CPPHY_MDIO_ST_LINK_WAIT;
            mdio->timeout = CPPHY_LINK_TO;
        }
#       if defined(CONFIG_AVM_POWERMETER)
        PowerManagmentRessourceInfo(powerdevice_ethernet, 0);
#       endif /*--- #if defined(CONFIG_AVM_POWERMETER) ---*/
        cpphy_mdio_event_dataupdate(mdio); /* State changed from LINKED */
        cpphy_mdio_mdix_delay(mdio);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpphy_mdio_default_state(cpphy_mdio_t *mdio) {
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void cpphy_mdio_tick(cpphy_mdio_t *mdio) {
    unsigned int last_state = mdio->state;

    mdio->event_last_update++;
    if(mdio->event_notify && (mdio->event_last_update > CPMAC_TICK_PER_SEC)) {
        /* Send each second information to interested parties */
        mdio->event_notify = 0;
        mdio->event_last_update = 0;
        cpmac_if_control_ind(mdio->cpmac_priv, CPMAC_CONTROL_IND_EVENT_UPDATE);
    }

    /* Update link information for switch products, if necessary */
    if(mdio->cpmac_switch) {
        if(!mdio->cpmac_irq) {
            /* Code is here, because this is faster than calling adm_check_link too often */
            unsigned int adm_status = ADM_GET_SERIAL_REG(mdio, REG_ADM_LC_STATUS0);

            if(adm_status != mdio->adm_status) {
                mdio->adm_status = adm_status;
                adm_check_link(mdio);
            }
        }
        if(--mdio->adm_power.timeout == 0) {
            adm_switch_port_power(mdio);
        }

    }

    switch(mdio->state) {
        case CPPHY_MDIO_ST_FINDING:
            cpphy_mdio_finding_state(mdio);
            break;
        case CPPHY_MDIO_ST_FOUND:
            cpphy_mdio_found_state(mdio);
            break;
        case CPPHY_MDIO_ST_PDOWN:
            cpphy_mdio_pdown_state(mdio);
            break;
        case CPPHY_MDIO_ST_NWAY_START:
            cpphy_mdio_nway_start_state(mdio);
            break;
        case CPPHY_MDIO_ST_NWAY_WAIT:
            cpphy_mdio_nway_wait_state(mdio);
            break;
        case CPPHY_MDIO_ST_LINK_WAIT:
            cpphy_mdio_link_wait_state(mdio);
            break;
        case CPPHY_MDIO_ST_LINKED:
            cpphy_mdio_linked_state(mdio);
            break;
        default:
            cpphy_mdio_default_state(mdio);
            break;
    }

    if(last_state != mdio->state) {
        struct net_device *p_dev = mdio->cpmac_priv->owner;

        /* Phy State Change Detected */
        if((mdio->state == CPPHY_MDIO_ST_LINKED) && !mdio->half_duplex) {
            /* to check: ok to assume always full duplex? */
            CPMAC_MACCONTROL(p_dev->base_addr) |= FULLDUPLEX;
        }
        DEB_INFO("cpphy_mdio_tick[%d]: state %u -> %u, MACCONTROL 0x%08X, %s\n", 
                 mdio->inst, last_state, mdio->state, CPMAC_MACCONTROL (p_dev->base_addr), 
                 (mdio->state == CPPHY_MDIO_ST_LINKED) ? "Linked" : "Not Linked");

        cpmac_if_control_ind(mdio->cpmac_priv, CPMAC_CONTROL_IND_MDIO_STATUS, 
                             mdio->state == CPPHY_MDIO_ST_LINKED);
    }
}


/*----------------------------------------------------------------------------------*\
  Start MDIO Negotiation
\*----------------------------------------------------------------------------------*/
void cpphy_mdio_init(cpphy_global_t *cpphy_global) {
    unsigned int port;

    cpphy_mdio_t *mdio = &cpphy_global->mdio;
    cpmac_priv_t *cpmac_priv = cpphy_global->cpmac_priv;

    mdio->cpmac_priv = cpmac_priv;
    for(port = 0; port < CPMAC_MAX_LED_HANDLES; port++) {
        mdio->cpmac_priv->led_handle[port] = 0;
    }

    /*  MII not setup, Setup initial condition  */
    if(cpmac_if_control_ind(mdio->cpmac_priv, CPMAC_CONTROL_IND_GET_INSTANCE_COUNT) == 1) {
        /* reset only during the first cpphy driver registration */
        avm_reset_device(MDIO_RESET_BIT, 10);
        msleep(2); /* wait after device out of reset */
    }

    MDIO_CR = MDIO_CR_ENABLE | MDIO_CR_CLKDIV((avm_get_clock(avm_clock_id_vbus) / CPPHY_CLK_FREQ) - 1);
    DEB_INFO("cpphy_mdio_init, vbus clk: %u\n", avm_get_clock(avm_clock_id_vbus));

#ifdef undef
    /* int pacing: chance to reduce (high) number of ints -> better caching + less redundancy */
    /* trick: with reduced clock in prescale register even smaller trigger values */
    /* than 2 Int/ms possible */
    /* currently no better performance */
    ohioint_ctrl_irq_pacing(OHIOINT_CPMAC0, 0, avm_get_clock(avm_clock_id_vbus) / (1000000 * 4 / 2), 2);
#endif

    /* design: must contain NWAY_HD100 or NWAY_HD10 */
    mdio->nway_mode = NWAY_AUTO|NWAY_HD10|NWAY_HD100|NWAY_FD10|NWAY_FD100;
    
    mdio->half_duplex = 0;
    mdio->slow_speed = 0;
    
    if(cpphy_global->high_phy) {
        /* to check */
        mdio->phy_num = 1;
        mdio->inst = 1;
    } else {
        mdio->phy_num = 31;
        mdio->inst = 0;
    }
    
    mdio->cpmac_switch = cpphy_global->cpmac_switch;
    mdio->cpmac_irq = 0;
    mdio->is_vdsl = cpphy_global->is_vdsl;
    if(mdio->cpmac_switch) {
        mdio->adm_status = 0;
        adm_init(mdio);
        adm_vlan_fbox_reset(mdio);
        /* wait for link up (switch is in auto mode) */
        mdio->state = CPPHY_MDIO_ST_LINK_WAIT;  
    } else {
        mdio->state = CPPHY_MDIO_ST_FINDING;
        mdio->timeout = CPPHY_FIND_TO;
        if(!cpphy_global->high_phy) {
            /* only internal (low) phy can be handled with GPIO_BIT_MII_MDIX! */
            mdio->nway_mode |= NWAY_AUTOMDIX;
        }
        avm_gpio_ctrl(GPIO_BIT_MII_MDIX, GPIO_PIN, GPIO_OUTPUT_PIN); 
        /* start with assumption: connected to pc with normal cable */
        cpphy_mdio_mdix_update(mdio->mdix);
        avm_gpio_out_bit(30, 0);   /* to check */
        cpphy_mdio_WriteEphyTestRegs(mdio, 31, 0x16, TNETD73XX_EPHY_TRIM_VALUE);
    }
}

