/*------------------------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*------------------------------------------------------------------------------------------*/
#ifndef _CPPHY_TYPES_H_
#define _CPPHY_TYPES_H_

#include <linux/avm_cpmac.h>
#include "cpmac_priority.h"

typedef struct {
  struct net_device *p_dev;
  cpmac_phy_handle_t phy_handle;
  cpmac_service_funcs_t service_funcs;
} dev_desc_t;

typedef enum {
    CPPHY_HW_ST_INITIALIZED = 1,
    CPPHY_HW_ST_OPENED
} cpphy_hw_state_t;

typedef enum {
    CPPHY_MDIO_ST_FINDING    = 2,
    CPPHY_MDIO_ST_FOUND      = 3,
    CPPHY_MDIO_ST_NWAY_START = 4,
    CPPHY_MDIO_ST_NWAY_WAIT  = 5,
    CPPHY_MDIO_ST_LINK_WAIT  = 6,
    CPPHY_MDIO_ST_LINKED     = 7,
    CPPHY_MDIO_ST_LOOPBACK   = 8,
    CPPHY_MDIO_ST_PDOWN      = 9
} cpphy_mdio_state_t;

typedef enum {
    CPPHY_SWITCH_MODE_16BIT = (1 << 0),
    CPPHY_SWITCH_MODE_READ  = (1 << 1),
    CPPHY_SWITCH_MODE_WRITE = (1 << 2)
} cpphy_switch_mode_t;

typedef struct {
    unsigned int  timeout;
    unsigned char current_on;
    unsigned char current_port;
    unsigned int  roundrobin;
    adm_phy_power_setup_t setup;
    unsigned char port_on[5];
} cpphy_adm_power;

typedef struct {
    cpphy_mdio_state_t             state;
    unsigned int                   pad_on; /* FIXME for testing */
    unsigned int                   nway_mode;
    unsigned int                   control;
    unsigned int                   timeout;
    unsigned int                   phy_num;
    unsigned int                   inst;
    cpmac_priv_t                  *cpmac_priv;
    unsigned int                   event_notify;
    unsigned int                   event_last_update;
    unsigned int                   cpmac_switch;
    unsigned int                   is_vdsl;
    unsigned int                   adm_status;
    cpphy_adm_power                adm_power;
    unsigned int                   linked;
    unsigned int                   half_duplex;
    unsigned int                   slow_speed;
    unsigned int                   mdix;
    unsigned int                   power_down;
    void                          *power_handle;
    cpphy_switch_mode_t            Mode;
    unsigned int                   cpmac_irq;
    struct avm_cpmac_config_struct cpmac_config;
    struct avm_new_switch_struct   switch_status;
    cpmac_switch_configuration_t  *switch_config;
    struct cpmac_event_struct      event_data;
} cpphy_mdio_t;

/* Defined in cpmac_priority.h */
/*--- typedef struct { ...  } cpphy_tcb_t; ---*/

typedef struct {
    unsigned int    HNext;
    unsigned int    BufPtr;
    unsigned int    Off_BLen;
    unsigned int    mode;
    void           *Next;
    struct sk_buff *skb;
} cpphy_rcb_t;

typedef struct cpphy_cppi_struct {
    cpphy_rcb_t            *RxPrevEnqueue;
    cpphy_rcb_t            *RxCurrDequeue;
    cpphy_rcb_t            *RxFirst;
    cpphy_rcb_t            *RxLast;
    int                     RxTeardownPending;
    char                   *RcbStart;
    cpphy_tcb_t            *TxPrevEnqueue;
    cpphy_tcb_t            *TxCurrDequeue;
    volatile cpphy_tcb_t   *TxFirst;
    volatile cpphy_tcb_t   *TxLast;
    unsigned int            FreeCount;
    volatile cpphy_tcb_t   *TxFirstFree;
    volatile cpphy_tcb_t   *TxLastFree;
    cpphy_tcb_prio_queues_t TxPrioQueues;
    atomic_t                dequeue_running;
    atomic_t                dma_send_running;
    volatile unsigned int   TxDmaActive;
    int                     TxTeardownPending;
    char                   *TcbStart;
    int                     TxOpen;
    int                     RxOpen;
    unsigned char           TxChannel;
    unsigned int            MaxNeedCount;
    unsigned int            CurrNeedCount;
    cpphy_hw_state_t        hw_state;
    unsigned int            hash1;
    unsigned int            hash2;
    cpmac_priv_t           *cpmac_priv;
    cpphy_mdio_t           *mdio;
} cpphy_cppi_t;

typedef struct {
    cpmac_priv_t *cpmac_priv;
    unsigned int  instance;
    unsigned int  high_phy;
    unsigned int  cpmac_switch;
    unsigned int  is_vdsl;
    cpphy_cppi_t  cppi;
    cpphy_mdio_t  mdio;
    unsigned int  is_used;
} cpphy_global_t;

typedef struct {
    cpphy_global_t    cpphy[CPMAC_MAX_PHY];
    unsigned int      timer_count;
    struct timer_list timer;
} cpmac_global_t;

extern cpphy_global_t cpmac_cpphy_global[];
extern cpmac_global_t cpmac_global;

#endif /*--- #ifndef _CPPHY_TYPES_H_ ---*/

