/*---------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*---------------------------------------------------------------------------*/

#if !defined (_ahcidesc_h_)
#define _ahcidesc_h_

#include "ahcidef.h"

/*---------------------------------------------------------------------------*\
  Heuristik, um die Transfertime abzuschtzen
  exact laut AVM USB Host Spec (rev 1.14):
  	Time <= (147 + (7*packetBytes*8/6))/8
  Abschtzung fr max 64 Bytes:
  	Time < 147/8 + 64*7/6 - 64 + packetBytes
  	Time < 18.375 + 74.67 - 64 + packetBytes
  	Time < 29.045 + packetBytes
\*---------------------------------------------------------------------------*/
#define AHCI_calcTransferTime(PackSize) ((PackSize) + 30)

/*---------------------------------------------------------------------------*\
	Descriptor DWORD1
\*---------------------------------------------------------------------------*/
typedef enum {	AHCI_TYPE_Setup = 0,
				AHCI_TYPE_Out, 
				AHCI_TYPE_In
} AHCI_TransferType;

#define AHCI_DWORD1_maxPacketSize_SHIFT		0
#define AHCI_DWORD1_transferType_SHIFT		10
#define AHCI_DWORD1_endpointNumber_SHIFT	12
#define AHCI_DWORD1_deviceAddress_SHIFT		16
#define AHCI_DWORD1_shortPAllowed_SHIFT		23
#define AHCI_DWORD1_period_SHIFT			24

#define AHCI_DWORD1_maxPacketSize_MASK	0x000003FF	// 10 Bits
#define AHCI_DWORD1_transferType_MASK	0x00000C00	// 2 Bits
#define AHCI_DWORD1_endpointNumber_MASK	0x0000F000	// 4 Bits
#define AHCI_DWORD1_deviceAddress_MASK	0x003F0000	// 7 Bits
#define AHCI_DWORD1_shortPAllowed_FLAG	0x00800000	// 1 Bit
#define AHCI_DWORD1_period_MASK			0xFF000000	// 8 Bits

#define AHCI_DWORD1_maxPacketSize(v) 	(((v) & AHCI_DWORD1_maxPacketSize_MASK) >> AHCI_DWORD1_maxPacketSize_SHIFT)
#define AHCI_DWORD1_transferType(v) 	(((v) & AHCI_DWORD1_transferType_MASK) >> AHCI_DWORD1_transferType_SHIFT)
#define AHCI_DWORD1_endpointNumber(v)	(((v) & AHCI_DWORD1_endpointNumber_MASK) >> AHCI_DWORD1_endpointNumber_SHIFT)
#define AHCI_DWORD1_deviceAddress(v) 	(((v) & AHCI_DWORD1_deviceAddress_MASK) >> AHCI_DWORD1_deviceAddress_SHIFT)
#define AHCI_DWORD1_shortPAllowed(v)	(((v) & AHCI_DWORD1_shortPAllowed_FLAG) != 0)
#define AHCI_DWORD1_period(v) 			(((v) & AHCI_DWORD1_period_MASK) >> AHCI_DWORD1_period_SHIFT)

/*---------------------------------------------------------------------------*\
	Descriptor DWORD2
\*---------------------------------------------------------------------------*/
typedef enum {	AHCI_Toggle0 =0,
				AHCI_Toggle1,
				AHCI_ToggleTake
} AHCI_ToggleMode;

#define AHCI_DWORD2_frameNumber_SHIFT		0
#define AHCI_DWORD2_transferTime_SHIFT		16
#define AHCI_DWORD2_errorCount_SHIFT		26
#define AHCI_DWORD2_remove_SHIFT			28
#define AHCI_DWORD2_isochron_SHIFT			29
#define AHCI_DWORD2_initialToggle_SHIFT		30
#define AHCI_DWORD2_takeToggle_SHIFT		31

#define AHCI_DWORD2_frameNumber_MASK		0x0000FFFF  // 16 Bits
#define AHCI_DWORD2_transferTime_MASK		0x007F0000	// 7 Bits
#define AHCI_DWORD2_errorCount_MASK			0x0C000000	// 2 Bits
#define AHCI_DWORD2_remove_FLAG				0x10000000	// 1 Bit
#define AHCI_DWORD2_isochron_FLAG			0x20000000	// 1 Bit
#define AHCI_DWORD2_initialToggle_FLAG		0x40000000	// 1 Bit
#define AHCI_DWORD2_takeToggle_FLAG			0x80000000	// 1 Bit

#define AHCI_DWORD2_frameNumber(v) 		(((v) & AHCI_DWORD2_frameNumber_MASK) >> AHCI_DWORD2_frameNumber_SHIFT)
#define AHCI_DWORD2_transferTime(v) 	(((v) & AHCI_DWORD2_transferTime_MASK) >> AHCI_DWORD2_transferTime_SHIFT)
#define AHCI_DWORD2_errorCount(v) 		(((v) & AHCI_DWORD2_errorCount_MASK) >> AHCI_DWORD2_errorCount_SHIFT)
#define AHCI_DWORD2_remove(v) 			(((v) & AHCI_DWORD2_remove_FLAG) != 0)
#define AHCI_DWORD2_isochron(v) 		(((v) & AHCI_DWORD2_isochron_FLAG) != 0)
#define AHCI_DWORD2_initialToggle(v)	(((v) & AHCI_DWORD2_initialToggle_FLAG) != 0)
#define AHCI_DWORD2_takeToggle(v)		(((v) & AHCI_DWORD2_takeToggle_FLAG) != 0)

/*---------------------------------------------------------------------------*\
	Descriptor DWORD3
\*---------------------------------------------------------------------------*/
#define AHCI_DWORD3_dataPointer_SHIFT		0
#define AHCI_DWORD3_transferSize_SHIFT		18

#define AHCI_DWORD3_dataPointer_MASK		0x0003FFFF	// 18 Bits
#define AHCI_DWORD3_transferSize_MASK		0xFFFC0000	// 14 Bits

#define AHCI_DWORD3_dataPointer(v) 		(((v) & AHCI_DWORD3_dataPointer_MASK) >> AHCI_DWORD3_dataPointer_SHIFT)
#define AHCI_DWORD3_transferSize(v)		(((v) & AHCI_DWORD3_transferSize_MASK) >> AHCI_DWORD3_transferSize_SHIFT)

/*---------------------------------------------------------------------------*\
	Descriptor DWORD4
\*---------------------------------------------------------------------------*/
#define AHCI_DWORD4_nextVertical_SHIFT			0
#define AHCI_DWORD4_completed_SHIFT				14
#define AHCI_DWORD4_errorCompleted_SHIFT		15
#define AHCI_DWORD4_nextHorizontal_SHIFT		16
#define AHCI_DWORD4_canceled_SHIFT				31

#define AHCI_DWORD4_nextVertical_MASK		0x00003FFF //14 Bits
#define AHCI_DWORD4_completed_FLAG			0x00004000 //1 Bit
#define AHCI_DWORD4_errorCompleted_FLAG		0x00008000 //1 Bit
#define AHCI_DWORD4_nextHorizontal_MASK		0x3FFF0000 //14 Bits
#define AHCI_DWORD4_canceled_FLAG			0x80000000 //1 Bit

#define AHCI_DWORD4_nextVertical(v) 	(((v) & AHCI_DWORD4_nextVertical_MASK) >> AHCI_DWORD4_nextVertical_SHIFT)
#define AHCI_DWORD4_completed(v) 		(((v) & AHCI_DWORD4_completed_FLAG) != 0)
#define AHCI_DWORD4_errorCompleted(v) 	(((v) & AHCI_DWORD4_errorCompleted_FLAG) != 0)
#define AHCI_DWORD4_nextHorizontal(v) 	(((v) & AHCI_DWORD4_nextHorizontal_MASK) >> AHCI_DWORD4_nextHorizontal_SHIFT)
#define AHCI_DWORD4_canceled(v) 		(((v) & AHCI_DWORD4_canceled_FLAG) != 0)

/*---------------------------------------------------------------------------*\
	Bulk IN
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_InitBulkInDesc (volatile AHCI_DWORD *pDesc, 
		unsigned Dev, unsigned EndPoint, unsigned PackSize,
		AHCI_FLAG SPa, AHCI_ToggleMode ToggleMode, 
		unsigned TransferSize) {
	
	assert (PackSize <= 64);
	assert (EndPoint < 16);
	assert (Dev <= 127);
	assert (TransferSize < (1 << 14));
	assert (ToggleMode <= 3);
	
	pDesc[0] = (PackSize << AHCI_DWORD1_maxPacketSize_SHIFT)
			 | (AHCI_TYPE_In << AHCI_DWORD1_transferType_SHIFT)
			 | (EndPoint << AHCI_DWORD1_endpointNumber_SHIFT)
			 | (Dev << AHCI_DWORD1_deviceAddress_SHIFT)
			 | (SPa << AHCI_DWORD1_shortPAllowed_SHIFT)
			 /* | (0 << AHCI_DWORD1_period_SHIFT)*/;
			
	pDesc[1] =	0
				/* | (0 << AHCI_DWORD2_frameNumber_SHIFT) */
				| (AHCI_calcTransferTime (PackSize) << AHCI_DWORD2_transferTime_SHIFT)
				/* | (0 << AHCI_DWORD2_isochron_SHIFT) */
				| (ToggleMode << AHCI_DWORD2_initialToggle_SHIFT);
				/* Take Toggle ist im Toggle Mode enthalten */
	
	pDesc[2] =	0 /*(0 << AHCI_DWORD3_dataPointer_SHIFT) */
				| (TransferSize << AHCI_DWORD3_transferSize_SHIFT);

	pDesc[3] =	0;

}

/*---------------------------------------------------------------------------*\
	Bulk OUT
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_InitBulkOutDesc (volatile AHCI_DWORD *pDesc, 
		unsigned Dev, unsigned EndPoint, unsigned PackSize,
		AHCI_ToggleMode ToggleMode, 
		unsigned TransferSize, AHCI_DATA_ADR DataPointer) {

	assert (PackSize <= 64);
	assert (EndPoint < 16);
	assert (Dev <= 127);
	assert (TransferSize < (1 << 14));
	assert (ToggleMode <= 3);
	assert (DataPointer < (1 << 18));
	
	pDesc[0] = (PackSize << AHCI_DWORD1_maxPacketSize_SHIFT)
			 | (AHCI_TYPE_Out << AHCI_DWORD1_transferType_SHIFT)
			 | (EndPoint << AHCI_DWORD1_endpointNumber_SHIFT)
			 | (Dev << AHCI_DWORD1_deviceAddress_SHIFT)
			 /* | (0 << AHCI_DWORD1_shortPAllowed_SHIFT) */
			 /* | (0 << AHCI_DWORD1_period_SHIFT) */;
			
	pDesc[1] =	0
				/* | (0 << AHCI_DWORD2_frameNumber_SHIFT) */
				| (AHCI_calcTransferTime (PackSize) << AHCI_DWORD2_transferTime_SHIFT)
				/* | (0 << AHCI_DWORD2_isochron_SHIFT) */
				| (ToggleMode << AHCI_DWORD2_initialToggle_SHIFT);
				/* Take Toggle ist im Toggle Mode enthalten */
	
	pDesc[2] = (DataPointer << AHCI_DWORD3_dataPointer_SHIFT)
			 | (TransferSize << AHCI_DWORD3_transferSize_SHIFT);

	pDesc[3] =	0;
}

/*---------------------------------------------------------------------------*\
	SETUP
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_InitSetupDesc (volatile AHCI_DWORD *pDesc,
		unsigned Dev, unsigned EndPoint,
		AHCI_DATA_ADR DataPointer) {

	pDesc[0] = (8 << AHCI_DWORD1_maxPacketSize_SHIFT)
			 /*| (AHCI_TYPE_Setup << AHCI_DWORD1_transferType_SHIFT)*/
			 | (EndPoint << AHCI_DWORD1_endpointNumber_SHIFT)
			 | (Dev << AHCI_DWORD1_deviceAddress_SHIFT)
			 /* | (0 << AHCI_DWORD1_shortPAllowed_SHIFT) */
			 /* | (0 << AHCI_DWORD1_period_SHIFT) */;
			
	pDesc[1] =	0
				/* | (0 << AHCI_DWORD2_frameNumber_SHIFT) */
				| (AHCI_calcTransferTime (8) << AHCI_DWORD2_transferTime_SHIFT)
				/* | (0 << AHCI_DWORD2_isochron_SHIFT) */
				| (AHCI_Toggle0 << AHCI_DWORD2_initialToggle_SHIFT);
				/* Take Toggle ist im Toggle Mode enthalten */
	
	pDesc[2] = (DataPointer << AHCI_DWORD3_dataPointer_SHIFT)
			 | (8 << AHCI_DWORD3_transferSize_SHIFT);

	pDesc[3] =	0;
}

/*---------------------------------------------------------------------------*\
	Interrupt
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_InitIntDesc (volatile AHCI_DWORD *pDesc, 
		unsigned FrameNumber, unsigned Period, 
		unsigned Dev, unsigned EndPoint, unsigned PackSize,
		AHCI_FLAG SPa, AHCI_ToggleMode ToggleMode,
		unsigned TransferSize) {

	assert (PackSize <= 64);
	assert (EndPoint < 16);
	assert (Dev <= 127);
	assert (TransferSize < (1 << 14));
	assert (ToggleMode <= 3);
	assert (Period <= 255);
	
	pDesc[0] = (PackSize << AHCI_DWORD1_maxPacketSize_SHIFT)
			 | (AHCI_TYPE_In << AHCI_DWORD1_transferType_SHIFT)
			 | (EndPoint << AHCI_DWORD1_endpointNumber_SHIFT)
			 | (Dev << AHCI_DWORD1_deviceAddress_SHIFT)
			 | (SPa << AHCI_DWORD1_shortPAllowed_SHIFT)
			 | (Period << AHCI_DWORD1_period_SHIFT);
			
	pDesc[1] = (FrameNumber << AHCI_DWORD2_frameNumber_SHIFT)
			 /* | (0 << AHCI_DWORD2_transferTime_SHIFT) */
			 /* | (0 << AHCI_DWORD2_isochron_SHIFT) */
			 | (ToggleMode << AHCI_DWORD2_initialToggle_SHIFT);
			 /* Take Toggle ist im Toggle Mode enthalten */
	
	pDesc[2] = 0 /* (DataPointer << AHCI_DWORD3_dataPointer_SHIFT) */
			 | (TransferSize << AHCI_DWORD3_transferSize_SHIFT);

	pDesc[3] =	0;
	
}

/*---------------------------------------------------------------------------*\
	Isochron IN
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_InitIsoInDesc (volatile AHCI_DWORD *pDesc, 
		unsigned FrameNumber, 
		unsigned Dev, unsigned EndPoint, unsigned PackSize,
		unsigned TransferSize) {

	assert (PackSize > 0);
	assert (PackSize <= 1280);
	assert (EndPoint < 16);
	assert (Dev <= 127);
	assert (TransferSize < (1 << 14));
	assert (PackSize && ((TransferSize % PackSize) == 0)); 
	
	pDesc[0] = (PackSize << AHCI_DWORD1_maxPacketSize_SHIFT)
			 | (AHCI_TYPE_In << AHCI_DWORD1_transferType_SHIFT)
			 | (EndPoint << AHCI_DWORD1_endpointNumber_SHIFT)
			 | (Dev << AHCI_DWORD1_deviceAddress_SHIFT)
			 /*  (0 << AHCI_DWORD1_shortPAllowed_SHIFT) */
			 | (1 << AHCI_DWORD1_period_SHIFT);
			
	pDesc[1] = (FrameNumber << AHCI_DWORD2_frameNumber_SHIFT)
			 /* | (0 << AHCI_DWORD2_transferTime_SHIFT) */
			 | (1 << AHCI_DWORD2_isochron_SHIFT)
			 /* | (ToggleMode << AHCI_DWORD2_initialToggle_SHIFT) */;
	
	pDesc[2] = 0 /* (DataPointer << AHCI_DWORD3_dataPointer_SHIFT) */
			 | (TransferSize << AHCI_DWORD3_transferSize_SHIFT);

	pDesc[3] =	0;
	
}

/*---------------------------------------------------------------------------*\
	Isochron OUT
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_InitIsoOutDesc (volatile AHCI_DWORD *pDesc, 
		unsigned FrameNumber, 
		unsigned Dev, unsigned EndPoint, unsigned PackSize,
		unsigned TransferSize, AHCI_DATA_ADR DataPointer) {

	assert (PackSize > 0);
	assert (PackSize <= 1280);
	assert (EndPoint < 16);
	assert (Dev <= 127);
	assert (TransferSize < (1 << 14));
	assert (PackSize && ((TransferSize % PackSize) == 0)); 
	assert (DataPointer < (1 << 18));
	
	pDesc[0] = (PackSize << AHCI_DWORD1_maxPacketSize_SHIFT)
			 | (AHCI_TYPE_Out << AHCI_DWORD1_transferType_SHIFT)
			 | (EndPoint << AHCI_DWORD1_endpointNumber_SHIFT)
			 | (Dev << AHCI_DWORD1_deviceAddress_SHIFT)
			 /*  (0 << AHCI_DWORD1_shortPAllowed_SHIFT) */
			 | (1 << AHCI_DWORD1_period_SHIFT);
			
	pDesc[1] = (FrameNumber << AHCI_DWORD2_frameNumber_SHIFT)
			 /* | (0 << AHCI_DWORD2_transferTime_SHIFT) */
			 | (1 << AHCI_DWORD2_isochron_SHIFT)
			 /* | (0 << AHCI_DWORD2_initialToggle_SHIFT) */;
	
	pDesc[2] = (DataPointer << AHCI_DWORD3_dataPointer_SHIFT)
			 | (TransferSize << AHCI_DWORD3_transferSize_SHIFT);

	pDesc[3] =	0;
	
}

#endif //_ahcimap_h_
