/*---------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*---------------------------------------------------------------------------*/

#if !defined (_ahcimem_h_)
#define _ahcimem_h_

/*---------------------------------------------------------------------------*\
	initializes AHCI MMU module
	IN uStartAddress: relative start address of descriptor memory
	IN size: total size of descriptor memory in bytes
	OUT != 0 --> Error
\*---------------------------------------------------------------------------*/
int AHCI_MemoryInit (unsigned uStartAddress, size_t size);

/*---------------------------------------------------------------------------*\
	deinitializes AHCI MMU module
	IN:		AHCI_MemoryInit called successfully
	OUT:	all ressources freed
\*---------------------------------------------------------------------------*/
void AHCI_MemoryDeinit (void);

/*---------------------------------------------------------------------------*\
	allocates memory in AHCI chip ram
	RETURN: relative start address (16 byte aligned) or 0
\*---------------------------------------------------------------------------*/
unsigned AHCI_MemoryAlloc (size_t size);

/*---------------------------------------------------------------------------*\
	frees previously allocated memory
	IN uStartAddress: relative start address of allocated memory (16 byte aligned)
\*---------------------------------------------------------------------------*/
void AHCI_MemoryFree (unsigned uStartAddress);

/*---------------------------------------------------------------------------*\
	frees first part of previously allocated memory
	IN uOldStartAddress: relative start address of allocated memory (16 byte aligned)
	IN uNewStartAddress: new relative start address (must be in range of allocation)
	RETURN: uNewStartAddress (16 byte aligned)
\*---------------------------------------------------------------------------*/
//unsigned AHCI_MemoryPartialFree (unsigned uOldStartAddress, unsigned uNewStartAddress);
//not implemented



void AHCI_MemorySetUrbPointer (unsigned Adr, urb_t *pUrb);
urb_t * AHCI_MemoryGetUrbPointer (unsigned Adr);


#endif //_ahcimem_h_



