/*---------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*---------------------------------------------------------------------------*/

#include "ahcidef.h"
#include "ahciport.h"
#include "ahcireg.h"
#include "ahciops.h"

#include "hub.h"

#ifdef CONFIG_AVM_POWER
#include <linux/avm_power.h>
#endif /*--- #ifdef CONFIG_AVM_POWER ---*/

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
unsigned char AHCI_MaxPortIndex = 0;
unsigned char AHCI_PortChangeFlags = 0;

AHCI_PortStatusStruct AHCI_Ports[CONFIG_AHCI_NUMPORTS];

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
void ahci_InitPorts (unsigned maxPortIndex) {
	unsigned i;
	info (maxPortIndex < CONFIG_AHCI_NUMPORTS);
	AHCI_MaxPortIndex = AHCI_MIN (maxPortIndex, CONFIG_AHCI_NUMPORTS-1);
	
	AHCI_PortChangeFlags = 0;
	for (i = 0; i <= AHCI_MaxPortIndex; i ++) {
		AHCI_DisablePort (i);
		AHCI_Ports[i].PortStatus = 0;
		AHCI_Ports[i].PortChange = 0;
	}
	ahci_UpdatePortStatus ();
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
static void ahci_UpdatePort (unsigned PortIndex) {
	unsigned PortStatus = AHCI_READ_REGISTER (AHCI_GetPortReg (PortIndex));
	unsigned char bChange = 0;

    unsigned short *pPortStatus = &AHCI_Ports[PortIndex].PortStatus;
	unsigned short *pPortChange = &AHCI_Ports[PortIndex].PortChange;

	PrintInfo ("UpdatePort Index %u: Status 0x%X\n", PortIndex, PortStatus);

	if ((*pPortStatus & USB_PORT_STAT_RESET) && !(PortStatus & AHCI_PCRR_portReset_FLAG)) {
		*pPortChange |= USB_PORT_STAT_C_RESET;
		*pPortStatus &= ~USB_PORT_STAT_RESET; //reset complete

		if (PortStatus & AHCI_PCRR_portEnabled_FLAG) {
			*pPortStatus |= USB_PORT_STAT_ENABLE; //reset complete
		}
		bChange = 1;
	}

    if (((*pPortStatus & USB_PORT_STAT_CONNECTION) != 0) != 
		((PortStatus & AHCI_PCRR_deviceConnected_FLAG) != 0)) {
			
		*pPortChange |= USB_PORT_STAT_C_CONNECTION;
		*pPortStatus ^= USB_PORT_STAT_CONNECTION; //connect /disconnect
		bChange = 1;
	}

	if ((*pPortStatus & USB_PORT_STAT_ENABLE) && !(PortStatus & AHCI_PCRR_portEnabled_FLAG)) {
		*pPortChange |= USB_PORT_STAT_C_ENABLE;
		*pPortStatus &= ~USB_PORT_STAT_ENABLE; //disable
		bChange = 1;
	}

	if ((*pPortStatus & USB_PORT_STAT_SUSPEND) && 
		!(PortStatus & (AHCI_PCRR_portSuspended_FLAG|AHCI_PCRR_portResume_FLAG))) {
			
		*pPortChange |= USB_PORT_STAT_SUSPEND;
		*pPortStatus &= ~USB_PORT_STAT_SUSPEND; //resume done
		bChange = 1;
	}
	
	if (bChange) {
		AHCI_PortChangeFlags |= (1 << (PortIndex+1));
		PrintInfo ("AHCI_PortChangeFlags = %x\n", AHCI_PortChangeFlags);
	}
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
void ahci_UpdatePortStatus (void) {
	unsigned i;
    for (i = 0; i <= AHCI_MaxPortIndex; i ++) {
        ahci_UpdatePort (i);
    }

#   ifdef CONFIG_AVM_POWER
    {
        static unsigned char bDevicePlugged = 0;
        unsigned char bDevicePluggedNew = 0;
        for (i = 0; i <= AHCI_MaxPortIndex; i ++) {
            bDevicePluggedNew |= ((AHCI_Ports[i].PortStatus & USB_PORT_STAT_CONNECTION) != 0);
        }
        if (bDevicePluggedNew) {
            if (!bDevicePlugged) {
                bDevicePlugged = 1;
                PrintInfo ("AHCI: Power Profile on\n");
                PowerManagmentActivatePowerMode ("usb_profile_on");
            }
        } else {
            if (bDevicePlugged) {
                bDevicePlugged = 0;
                PrintInfo ("AHCI: Power Profile off\n");
                PowerManagmentActivatePowerMode ("usb_profile_off");
            }
        }
    }
#   endif /* --- #ifdef CONFIG_AVM_POWER ---*/

}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
