/*---------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*---------------------------------------------------------------------------*/

#ifndef _ahciport_h_
#define _ahciport_h_

#include "ahcireg.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
#pragma pack (1)
	typedef struct {
		unsigned short PortStatus;
		unsigned short PortChange;
	} AHCI_PortStatusStruct;
#pragma pack ()

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
extern AHCI_PortStatusStruct AHCI_Ports[CONFIG_AHCI_NUMPORTS];

extern unsigned char AHCI_PortChangeFlags;
extern unsigned char AHCI_MaxPortIndex;

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
void ahci_InitPorts (unsigned maxPortIndex);
void ahci_UpdatePortStatus (void);

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
PAHCI_REGISTER AHCI_GetPortReg (unsigned PortIndex) {
	assert (PortIndex <= AHCI_MaxPortIndex);
	return pAHCI_PortControlRegs + PortIndex;
}

/*---------------------------------------------------------------------------*\
   Ports Control Register
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
AHCI_DWORD AHCI_GetPortStatus (unsigned PortIndex) {
	PAHCI_REGISTER pPortControlReg = AHCI_GetPortReg (PortIndex);

	return AHCI_READ_REGISTER (pPortControlReg);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_DisablePort (unsigned PortIndex) {
	PAHCI_REGISTER pPortControlReg = AHCI_GetPortReg (PortIndex);

	info ((AHCI_READ_REGISTER (pPortControlReg) & AHCI_PCRR_deviceConnected_FLAG) != 0);
	AHCI_WRITE_REGISTER (pPortControlReg, AHCI_PCRW_disablePort_FLAG);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_ResetPort (unsigned PortIndex) {
	PAHCI_REGISTER pPortControlReg = AHCI_GetPortReg (PortIndex);

	assert ((AHCI_READ_REGISTER (pPortControlReg) & AHCI_PCRR_deviceConnected_FLAG) != 0);
	AHCI_WRITE_REGISTER (pPortControlReg, AHCI_PCRW_resetPort_FLAG);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_SuspendPort (unsigned PortIndex) {
	PAHCI_REGISTER pPortControlReg = AHCI_GetPortReg (PortIndex);

	assert ((AHCI_READ_REGISTER (pPortControlReg) & AHCI_PCRR_portEnabled_FLAG) != 0);
	AHCI_WRITE_REGISTER (pPortControlReg, AHCI_PCRW_suspendPort_FLAG);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_ResumePort (unsigned PortIndex) {
	PAHCI_REGISTER pPortControlReg = AHCI_GetPortReg (PortIndex);

	assert ((AHCI_READ_REGISTER (pPortControlReg) & AHCI_PCRR_portSuspended_FLAG) != 0);
	AHCI_WRITE_REGISTER (pPortControlReg, AHCI_PCRW_resumePort_FLAG);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
AHCI_INLINE_PREFIX
void AHCI_ClearRemoteWakup (unsigned PortIndex) {
	PAHCI_REGISTER pPortControlReg = AHCI_GetPortReg (PortIndex);

	assert ((AHCI_READ_REGISTER (pPortControlReg) & AHCI_PCRR_remoteWakeup_FLAG) != 0);
	AHCI_WRITE_REGISTER (pPortControlReg, AHCI_PCRW_clearRemoteWakeup_FLAG);
}

#endif

