/*---------------------------------------------------------------------------*\
 *   
 *   Copyright (C) 2006 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
\*---------------------------------------------------------------------------*/

#include "ahcidriver.h"
#include "ahcidef.h"
#include "ahcireg.h"
#include "ahcidesc.h"
#include "ahciops.h"
#include "ahcirx.h"

#include "ahcimem.h"
#include "ahciport.h"

#include "usb.h"
#include "hcd.h"
#include "hub.h"

#ifdef CONFIG_AVM_POWERMETER
#include <linux/avm_power.h>
#endif /*--- #ifdef CONFIG_AVM_POWERMETER ---*/

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
static int ahci_set_rh_timer (urb_t * urb);
static int g_ahci_rh_ports_active = 0;

struct work_struct ahci_rh_work_struct;

/*-------------------------------------------------------------------
 Virtual Root Hub
 -------------------------------------------------------------------*/
static unsigned char ahci_root_hub_device_descriptor[] =
{
	0x12,			/*  __u8  bLength; */
	0x01,			/*  __u8  bDescriptorType; Device */
	0x10,			/*  __u16 bcdUSB; v1.1 */
	0x01,
	0x09,			/*  __u8  bDeviceClass; HUB_CLASSCODE */
	0x00,			/*  __u8  bDeviceSubClass; */
	0x00,			/*  __u8  bDeviceProtocol; */
	0x08,			/*  __u8  bMaxPacketSize0; 8 Bytes */
	0x00,			/*  __u16 idVendor; */
	0x00,
	0x00,			/*  __u16 idProduct; */
	0x00,
	0x00,			/*  __u16 bcdDevice; */
	0x00,
	0x00,			/*  __u8  iManufacturer; */
	0x02,			/*  __u8  iProduct; */
	0x01,			/*  __u8  iSerialNumber; */
	0x01			/*  __u8  bNumConfigurations; */
};

/* Configuration descriptor */
static unsigned char ahci_root_hub_config_descriptor[] =
{
	0x09,			/*  __u8  bLength; */
	0x02,			/*  __u8  bDescriptorType; Configuration */
	0x19,			/*  __u16 wTotalLength; */
	0x00,
	0x01,			/*  __u8  bNumInterfaces; */
	0x01,			/*  __u8  bConfigurationValue; */
	0x00,			/*  __u8  iConfiguration; */
	0x40,			/*  __u8  bmAttributes;
				   Bit 7: Bus-powered, 6: Self-powered, 5 Remote-wakwup, 4..0: resvd */
	0x00,			/*  __u8  MaxPower; */

     /* interface */
	0x09,			/*  __u8  if_bLength; */
	0x04,			/*  __u8  if_bDescriptorType; Interface */
	0x00,			/*  __u8  if_bInterfaceNumber; */
	0x00,			/*  __u8  if_bAlternateSetting; */
	0x01,			/*  __u8  if_bNumEndpoints; */
	0x09,			/*  __u8  if_bInterfaceClass; HUB_CLASSCODE */
	0x00,			/*  __u8  if_bInterfaceSubClass; */
	0x00,			/*  __u8  if_bInterfaceProtocol; */
	0x00,			/*  __u8  if_iInterface; */

     /* endpoint */
	0x07,			/*  __u8  ep_bLength; */
	0x05,			/*  __u8  ep_bDescriptorType; Endpoint */
	0x81,			/*  __u8  ep_bEndpointAddress; IN Endpoint 1 */
	0x03,			/*  __u8  ep_bmAttributes; Interrupt */
	0x08,			/*  __u16 ep_wMaxPacketSize; 8 Bytes */
	0x00,
	0xff			/*  __u8  ep_bInterval; 255 ms */
};

static unsigned char ahci_root_hub_hub_descriptor[] =
{
	0x09,			/*  __u8  bLength; */
	0x29,			/*  __u8  bDescriptorType; Hub-descriptor */
	0x00,			/*  __u8  bNbrPorts; */
	0x00,			/* __u16  wHubCharacteristics; */
	0x00,
	0x01,			/*  __u8  bPwrOn2pwrGood; 2ms */
	0x00,			/*  __u8  bHubContrCurrent; 0 mA */
	0x00,			/*  __u8  DeviceRemovable; *** 7 Ports max *** */
	0xff			/*  __u8  PortPwrCtrlMask; *** 7 ports max *** */
};

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
struct pt_regs		dummy_regs;		/* Also used by ahciurb.c! */


/* TAKEN FROM A GPL MODULE --- FIXME */

static int ascii2utf (char * ascii, __u8 * utf, int utfmax) {
	int retval;

	for (retval = 0; *ascii && utfmax > 1; utfmax -= 2, retval += 2) {
		*utf++ = *ascii++ & 0x7f;
		*utf++ = 0;
	}
	return retval;
} /* ascii2utf */

static int usb_root_hub_string (int id, int serial, char *type, __u8 *data, int len) {
	char	buf[30];

	assert (len > (2 * (sizeof (buf) + 1)));
	assert (strlen (type) <= 8);
	
	// language ids
	if (id == 0) {
		*data++ = 4; *data++ = 3;       /* 4 bytes data */
		*data++ = 0; *data++ = 0;       /* some language id */
		return 4;

	// serial number
	} else if (id == 1) {
		sprintf (buf, "%x", serial);
		
	// product description
	} else if (id == 2) {
		sprintf (buf, "USB %s Root Hub", type);
	
	// id 3 == vendor description
	
	// unsupported IDs --> "stall"
	} else
		return 0;

	data[0] = 2 + ascii2utf (buf, data + 2, len - 2);
	data[1] = 3;
	return data[0];
} /* usb_root_hub_string */

static void ahci_rh_timer_callback (unsigned long context);
static void ahci_rh_work (void *pContext);
/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
int ahci_init_root_hub (struct ahci_struct * pahci) {
	struct usb_device * usb_rh;

	assert (pahci != 0);
	assert (pahci->bus != 0);
	
	ahci_root_hub_hub_descriptor[2] = AHCI_MaxPortIndex+1;
	
	memset (&dummy_regs, 0, sizeof (dummy_regs));
	
	PrintInfo ("alloc AHCI root hub\n");
	usb_rh = usb_alloc_dev (NULL, ahci.bus, 0);
	if (NULL == usb_rh) {
		return -ENOMEM;
	}

	PrintInfo ("connect AHCI root hub\n");
	/*
	 * Does not exist anymore in 2.6 :
	 *
	 * usb_connect (usb_dev);
	 *
	 * Using this instead:
	 */
	{
	ahci.rh_address = 1;
        usb_rh->state = USB_STATE_ADDRESS;
	usb_rh->speed = USB_SPEED_FULL;
	usb_rh->devnum = ahci.rh_address;
	pahci->bus->devnum_next = 2;
	usb_rh->ep0.desc.wMaxPacketSize = __constant_cpu_to_le16(64);
	usb_get_device_descriptor(usb_rh, USB_DT_DEVICE_SIZE);
	}

	init_timer (&ahci.rh_timer);
	pahci->bus->root_hub = usb_rh;

	INIT_WORK (&ahci_rh_work_struct, ahci_rh_work, 0);
	schedule_delayed_work (&ahci_rh_work_struct, 2*HZ);
	return 0;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
void ahci_deinit_root_hub (struct ahci_struct * pahci) {
	//usb_disconnect sets pointer to 0
	assert (pahci != 0);
	info (pahci->bus != NULL);
	if ((pahci->bus != NULL) && (pahci->bus->root_hub != NULL)) {
		cancel_delayed_work (&ahci_rh_work_struct);
		usb_disconnect (&pahci->bus->root_hub);
		del_timer (&pahci->rh_timer);
	}
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
static void ahci_rh_work (void * pContext)
{
    PrintInfo ("work callback:new device AHCI root hub\n");
	if (ahci.bus->root_hub != NULL) {
		if (usb_new_device (ahci.bus->root_hub) != 0) {
			PrintError ("error on new device AHCI root hub\n");
			ahci.bus->root_hub = NULL;
		}
	}
	
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
static void ahci_rh_timer_callback (unsigned long context)
{
	int len= 0; 
	

	urb_t * urb = (urb_t *) context;
	if (urb == 0) {
		return;
	}
 
	//PrintInfo ("AHCI RH Timer Callback\n");
	assert (urb->dev->bus->hcpriv == &ahci);
    info (ahci.rh_urb == urb);
	if (ahci.rh_urb != urb) {
		return;
	}
    
#   ifdef CONFIG_AVM_POWERMETER
    {
        /* first Port Only */
        static unsigned char ucOldPower = 255;
        unsigned char ucNewPower = 0;
        struct usb_device *childdev = ahci.rh_urb->dev->children[0];
        if (childdev != NULL) {
            ucNewPower = (childdev->actconfig != NULL)
                            ? childdev->actconfig->desc.bMaxPower
                            : 50;/* default: 50 means 100 mA*/
        }
        if (ucOldPower != ucNewPower) {
            ucOldPower = ucNewPower;
            PrintInfo ("AHCI: Power changed to %u mA\n", ucNewPower*2);
            PowerManagmentRessourceInfo(powerdevice_usb_host, ucNewPower*2);
       }
    }
#endif  /*--- #ifdef CONFIG_AVM_POWERMETER ---*/

    

	ahci.rh_urb = 0;

	info (g_ahci_rh_ports_active);
	//Ports aktivieren
	g_ahci_rh_ports_active = 1;

	info (AHCI_PortChangeFlags == 0);
	if (AHCI_PortChangeFlags != 0) {
		if (urb->transfer_buffer_length >= 1) {
			unsigned char PortChangeFlags = AHCI_PortChangeFlags;
			AHCI_PortChangeFlags &= ~PortChangeFlags;
			
			((char *)urb->transfer_buffer)[0] = PortChangeFlags;
			len = 1;
			PrintInfo ("Hub Int PortChangeFlags = %X\n", PortChangeFlags);
		}
	}

	if (len > 0) {
		PrintInfo ("TIMER: root hub Int urb %p completion\n", urb);
		urb->actual_length = len;
		urb->status = USB_ST_NOERROR;
		if (urb->complete) {
			/*
			 * NOTE: This API requires to transfer a struct pt_regs
			 *       here, but there seems to be no safe way to get
			 *       a valid pt_regs at runtime (on all platforms),
			 *       so we transfer a dummy structure and issue the
			 *       warning: "DO NOT USE PT_REGS IN THE COMPLETION
			 *       HANDLER!!!"
			 */
			urb->complete (urb, &dummy_regs);
			//wkrubeck
			//new for 2.6, not again after complete
			return;
		}
	}
	
	ahci_set_rh_timer (urb);
	
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
int ahci_rh_unlink_urb (urb_t * urb) {
	assert (urb != 0);
	assert (urb->dev->bus->hcpriv == &ahci);
	assert (urb->hcpriv == 0);

	PrintInfo ("AHCI RH Unlink Urb\n");

	info (ahci.rh_urb == urb);
	if (ahci.rh_urb != urb) {
		return -EINVAL;
	}
	
	del_timer (&ahci.rh_timer);
	ahci.rh_urb = 0;
	
	if (urb->transfer_flags & URB_ASYNC_UNLINK) {
		urb->status = -ECONNRESET;
		if (urb->complete) {
			urb->complete (urb, &dummy_regs);	/* See comment in line #195! */
		}
	} else {
		urb->status = -ENOENT;
	}	

	return 0;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
static int ahci_set_rh_timer (urb_t * urb) {
	//PrintInfo ("ahci_set_rh_timer (urb = %p)\n", urb);

	ahci.rh_urb =  urb;
	init_timer (&ahci.rh_timer);
	
	ahci.rh_timer.function = ahci_rh_timer_callback;
	ahci.rh_timer.data = (unsigned long) urb;
	
	info (g_ahci_rh_ports_active);
	if (g_ahci_rh_ports_active) {
		// 255 msec later
		ahci.rh_timer.expires = (jiffies + 1) + (HZ * 255) / 1000;
	} else {
		PrintInfo ("ahci_set_rh_timer first time\n");
		// initial delay: activate ports after 5 seconds
		ahci.rh_timer.expires = (jiffies + 1) + (HZ * 5);
	}

	add_timer (&ahci.rh_timer);
	
	return 0;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
int ahci_rh_submit_urb (urb_t * urb) {
	unsigned char			tmp_buf[12];
	unsigned int			pipe		= urb->pipe;
	unsigned			len		= 0;
	unsigned char *			copy_buf	= tmp_buf;
	int				status		= USB_ST_STALL;
	
	/* 2.4: devrequest   2.6: struct usb_ctrlrequest */
	struct usb_ctrlrequest *	cmd		= (struct usb_ctrlrequest *) urb->setup_packet;

	//PrintInfo ("ahci_root_hub request\n");
	
	if (usb_pipeint (pipe)) {
		PrintInfo ("ahci_root_hub Int\n");
		urb->status = USB_ST_URB_PENDING;

		ahci_set_rh_timer (urb);
		
		return USB_ST_NOERROR;
	}

	//PrintInfo ("ahci_root_hub request %X with type %X\n", cmd->request, cmd->requesttype);
	
	switch ((cmd->bRequest<<8) | cmd->bRequestType) {
		case (USB_REQ_GET_STATUS<<8)|USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_DEVICE:
			PrintInfo ("USB_REQ_GET_STATUS dev\n");
			len = 2;
			tmp_buf[0] = 1;	/* Self-Powered */
			tmp_buf[1] = 0;
			status = USB_ST_NOERROR;
			break;
			
		case (USB_REQ_GET_STATUS<<8)|USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_INTERFACE:
			PrintInfo ("USB_REQ_GET_STATUS Interface\n");
			len = 2;
			tmp_buf[0] = 0;
			tmp_buf[1] = 0;
			status = USB_ST_NOERROR;
			break;
			
		case (USB_REQ_GET_STATUS<<8)|USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_ENDPOINT:
			PrintInfo ("USB_REQ_GET_STATUS EndPoint\n");
			len = 2;
			tmp_buf[0] = 0;
			tmp_buf[1] = 0;
			status = USB_ST_NOERROR;
			break;
		
		case (USB_REQ_GET_STATUS<<8)|USB_DIR_IN|USB_RT_HUB:
			PrintInfo ("USB_REQ_GET_STATUS Hub\n");
			len = 4;
			tmp_buf[0] = 0;
			tmp_buf[1] = 0;
			tmp_buf[2] = 0;
			tmp_buf[3] = 0;
			status = USB_ST_NOERROR;
			break;
			
		case (USB_REQ_GET_STATUS<<8)|USB_DIR_IN|USB_RT_PORT: 
			{
				unsigned short PortIndex = cmd->wIndex -1;
				PrintInfo ("USB_REQ_GET_STATUS Port %u\n",cmd->wIndex);

				if (PortIndex > AHCI_MaxPortIndex) break;
				
				info (AHCI_Ports[PortIndex].PortStatus & USB_PORT_STAT_POWER);
				if ((AHCI_Ports[PortIndex].PortStatus & USB_PORT_STAT_POWER) == 0) {
					*(unsigned *) tmp_buf = 0;
				} else {
					info (g_ahci_rh_ports_active);
					*(unsigned *) tmp_buf = 
						g_ahci_rh_ports_active
						? ((AHCI_Ports[PortIndex].PortStatus) | (AHCI_Ports[PortIndex].PortChange << 16))
						: USB_PORT_STAT_POWER;
				}

				len = 4;
				PrintInfo ("PortStatus and change %08X\n", *(unsigned *) tmp_buf);
				status = USB_ST_NOERROR;
			}
			break;
		
		case (USB_REQ_CLEAR_FEATURE<<8)|USB_DIR_OUT|USB_TYPE_STANDARD|USB_RECIP_DEVICE:
			PrintInfo ("USB_REQ_CLEAR_FEATURE Dev with value 0x%04X\n", cmd->wValue);
			break;
			
		case (USB_REQ_CLEAR_FEATURE<<8)|USB_DIR_OUT|USB_RT_HUB:
			PrintInfo ("USB_REQ_CLEAR_FEATURE Hub with value 0x%04X\n", cmd->wValue);
			break;
			
		case (USB_REQ_CLEAR_FEATURE<<8)|USB_DIR_OUT|USB_RT_PORT:
			{
				unsigned short PortIndex = cmd->wIndex -1;
				PrintInfo ("USB_REQ_CLEAR_FEATURE Port 0x%04X\n", cmd->wIndex);

				if (PortIndex > AHCI_MaxPortIndex) break;

				switch (cmd->wValue) {
					case USB_PORT_FEAT_ENABLE:
						PrintInfo ("USB_REQ_CLEAR_FEATURE USB_PORT_FEAT_ENABLE\n");
						//prevents C_PORT_ENABLE
						AHCI_Ports[PortIndex].PortStatus &= ~USB_PORT_STAT_ENABLE;
						AHCI_DisablePort (PortIndex);

						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_SUSPEND:
						PrintInfo ("USB_REQ_CLEAR_FEATURE USB_PORT_FEAT_SUSPEND\n");
						AHCI_ResumePort (PortIndex);
						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_POWER:
						PrintInfo ("USB_REQ_CLEAR_FEATURE USB_PORT_FEAT_POWER\n");
						AHCI_Ports[PortIndex].PortStatus = 0;
						AHCI_Ports[PortIndex].PortChange = 0;
						AHCI_DisablePort (PortIndex);
						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_C_CONNECTION:
						PrintInfo ("USB_REQ_CLEAR_FEATURE USB_PORT_FEAT_C_CONNECTION\n");
						AHCI_Ports[PortIndex].PortChange &= ~USB_PORT_STAT_C_CONNECTION;
						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_C_RESET:
						PrintInfo ("USB_REQ_CLEAR_FEATURE USB_PORT_FEAT_C_RESET\n");
						AHCI_Ports[PortIndex].PortChange &= ~USB_PORT_STAT_C_RESET;
						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_C_ENABLE:
						PrintInfo ("USB_REQ_CLEAR_FEATURE USB_PORT_FEAT_C_ENABLE\n");
						AHCI_Ports[PortIndex].PortChange &= ~USB_PORT_STAT_C_ENABLE;
						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_C_SUSPEND:
						PrintInfo ("USB_REQ_CLEAR_FEATURE USB_PORT_FEAT_C_SUSPEND\n");
						AHCI_Ports[PortIndex].PortChange &= ~USB_PORT_STAT_C_SUSPEND;
						status = USB_ST_NOERROR;
						break;

					//case USB_PORT_FEAT_C_OVER_CURRENT:break;

				}
			}
			break;
		
		case (USB_REQ_SET_FEATURE<<8)|USB_DIR_OUT|USB_RT_PORT:
			{
				unsigned short PortIndex = cmd->wIndex -1;

				PrintInfo ("USB_SET_FEATURE Port %u with value 0x%04X\n", cmd->wIndex, cmd->wValue);
				if (PortIndex > AHCI_MaxPortIndex) break;

				switch (cmd->wValue) {
					case USB_PORT_FEAT_RESET:
						AHCI_Ports[PortIndex].PortStatus |= USB_PORT_STAT_RESET;
						AHCI_ResetPort (PortIndex);
						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_SUSPEND:
						AHCI_Ports[PortIndex].PortStatus |= USB_PORT_STAT_SUSPEND;
						AHCI_SuspendPort (PortIndex);
						status = USB_ST_NOERROR;
						break;

					case USB_PORT_FEAT_POWER:
						AHCI_Ports[PortIndex].PortStatus |= USB_PORT_STAT_POWER;
						status = USB_ST_NOERROR;
						break;
				}
			}
			break;

		case (USB_REQ_SET_ADDRESS<<8)|USB_DIR_OUT|USB_TYPE_STANDARD|USB_RECIP_DEVICE:
			PrintInfo ("!!!! USB_SET_ADDRESS not needed in Kernel 2.6\n");
			assert (0);
			break;
		
		case (USB_REQ_GET_DESCRIPTOR<<8)|USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_DEVICE:
			//PrintInfo ("USB_REQ_GET_DESCRIPTOR standard with value 0x%04X\n", cmd->wValue);
			switch (cmd->wValue >> 8) {
				case USB_DT_DEVICE:
						PrintInfo ("USB_REQ_GET_DESCRIPTOR dev\n");
						copy_buf = ahci_root_hub_device_descriptor;
						len = sizeof (ahci_root_hub_device_descriptor);
						status = USB_ST_NOERROR;
						break;

				case USB_DT_CONFIG:
						PrintInfo ("USB_REQ_GET_DESCRIPTOR conf\n");
						copy_buf = ahci_root_hub_config_descriptor;
						len = sizeof (ahci_root_hub_config_descriptor);
						status = USB_ST_NOERROR;
						break;

				case USB_DT_STRING:
						//PrintInfo ("USB_REQ_GET_DESCRIPTOR string\n");
					copy_buf = urb->transfer_buffer;
					len = usb_root_hub_string (
						cmd->wValue & 0xff, (int) AHCI_BaseAddress, 
						"AHCI", copy_buf, cmd->wLength
					);
					if (len > 0) {
						status = USB_ST_NOERROR;
					}
					break;
					
				default:
					PrintError ("Error: USB_REQ_GET_DESCRIPTOR value 0x%04X not supported\n", cmd->wValue);
					break;
			}
			break;
					
		case (USB_REQ_GET_DESCRIPTOR<<8)|USB_DIR_IN|USB_RT_HUB:
			PrintInfo ("USB_REQ_GET_DESCRIPTOR HUB\n");
			copy_buf = ahci_root_hub_hub_descriptor;
			len = sizeof (ahci_root_hub_hub_descriptor);
			
			status = USB_ST_NOERROR;
			break;
			
		//case (USB_REQ_SET_DESCRIPTOR<<8)|USB_DIR_OUT|USB_TYPE_STANDARD|USB_RECIP_DEVICE:
		//	break;
			
		case (USB_REQ_GET_CONFIGURATION<<8)|USB_DIR_IN|USB_TYPE_STANDARD|USB_RECIP_DEVICE:
			PrintInfo ("USB_REQ_GET_CONFIG\n");
			copy_buf = tmp_buf;
			tmp_buf[0] = 1;
			len = 1;
			status = USB_ST_NOERROR;
			break;
			
		case (USB_REQ_SET_CONFIGURATION<<8)|USB_DIR_OUT|USB_TYPE_STANDARD|USB_RECIP_DEVICE:
			PrintInfo ("USB_REQ_SET_CONFIG\n");
			status = USB_ST_NOERROR;
			break;

		//case (USB_REQ_GET_INTERFACE<<8):
		//	break;
			
		//case USB_REQ_SET_INTERFACE<<8:
		//	break;
			
		//case (USB_REQ_SYNCH_FRAME<<8):
		//	break;
			
		default:
			PrintError ("Error: ahci_root_hub request %X with type %X not supported\n", cmd->bRequest, cmd->bRequestType);
	}
	
	if (len > (urb->transfer_buffer_length)) {
		len = urb->transfer_buffer_length;
	}
	if (copy_buf != urb->transfer_buffer) {
		memcpy (urb->transfer_buffer, copy_buf, len);
	}
	
  	urb->actual_length = len;
  	
	urb->status = status;
	urb->hcpriv = NULL;
	//usb_dec_dev_use (usb_dev);
	urb->dev = NULL;
	
	if (urb->complete != 0) {
		urb->complete (urb, &dummy_regs);	/* See comment in line #195! */
	}
	
	info (urb->status == USB_ST_NOERROR);
	info (urb->actual_length >= 1 || urb->transfer_buffer_length == 0);
	
	return urb->status;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/
