/*
 *  linux/include/asm-arm/arch-davinci/irq.h
 *
 *  BRIEF MODULE DESCRIPTION
 *      DAVINCI Virtual irq definitions
 *
 *  Copyright (C) 2006 Texas Instruments.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_IRQ_H
#define __ASM_ARCH_IRQ_H

/**************************************************************************
 * Included Files
 **************************************************************************/

/**************************************************************************
 * Global Function Prototypes
 **************************************************************************/
typedef struct intc_registers_t {
	unsigned int fiq0;	/* 0x0 */
	unsigned int fiq1;	/* 0x4 */
	unsigned int irq0;	/* 0x8 */
	unsigned int irq1;	/* 0xC */
	unsigned int fiqentry;	/* 0x10 */
	unsigned int irqentry;	/* 0x14 */
	unsigned int eint0;	/* 0x18 */
	unsigned int eint1;	/* 0x1C */
	unsigned int inctl;	/* 0x20 */
	unsigned int eabase;	/* 0x24 */
	unsigned int resv1;	/* 0x28 */
	unsigned int resv2;	/* 0x2C */
	unsigned int intpri0;	/* 0x30 */
	unsigned int intpri1;	/* 0x34 */
	unsigned int intpri2;	/* 0x38 */
	unsigned int intpri3;	/* 0x3C */
	unsigned int intpri4;	/* 0x40 */
	unsigned int intpri5;	/* 0x44 */
	unsigned int intpri6;	/* 0x48 */
	unsigned int intpri7;	/* 0x4C */
} intc_registers;

/****************************************************
 * DaVinci Interrupt numbers
 ****************************************************/

enum _davinci_irqs {
    vdint0 = 0,
    vdint1 = 1,
    vdint2 = 2,
    histnt = 3,
    h3aint = 4,
    prvuint = 5,
    rszint = 6,
    /* 7 */
    vencint = 8,
    asqint = 9,
    imxint = 10,
    vlcdint = 11,
    /* 12 */
    emacint = 13,
    /* 14 */
    /* 15 */
    edma3cc_int0 = 16,
    edma3cc_errint = 17,
    edma3cc_errint0 = 18,
    edma3cc_errint1 = 19,
    pscint = 20,
    /* 21 */
    ideint = 22,
    /* 23 */
    aspxint = 24,
    asprint = 25,
    mmcint = 26,
    sdioint = 27,
    /* 28 */
    ddrint = 29,
    emifaint = 30,
    vlqint = 31,
    tint0 = 32,
    tint1 = 33,
    tint2 = 34,
    tint3 = 35,
    pwmint0 = 36,
    pwmint1 = 37,
    pwmint2 = 38,
    i2cint = 39,
    uartint0 = 40,
    uartint1 = 41,
    uartint2 = 42,
    spint0 = 43,
    spint1 = 44,
    /* 45 */
    dsp2arm0 = 46,
    dsp2arm1 = 47,
    gpio0 = 48,
    gpio1 = 49,
    gpio2 = 50,
    gpio3 = 51,
    gpio4 = 52,
    gpio5 = 53,
    gpio6 = 54,
    gpio7 = 55,
    gpiobnk0 = 56,
    gpiobnk1 = 57,
    gpiobnk2 = 58,
    gpiobnk3 = 59,
    gpiobnk4 = 60,
    commtx = 61,
    commrx = 62,
    emuint = 63
};


#endif				/* __ASM_ARCH_IRQ_H */
